from RSTAB.enums import MemberNonlinearityType
from RSTAB.initModel import Model, clearAttributes, ConvertToDlString, deleteEmptyAttributes

class MemberNonlinearity():
    def __init__(self,
                 no: int = 1,
                 members: str = "",
                 nonlinearity_type = MemberNonlinearityType.TYPE_FAILURE_IF_TENSION,
                 parameters = None,
                 comment: str = '',
                 params: dict = None,
                 model = Model):
        """
        Args:
            no (int): Member Nonlinearity Tag
            members (str): Assigned Members
            nonlinearity_type (enum): Member Nonlinearity Type Enumeration Item
            parameters (list): Nonlinearity Parameters
            comment (str, optional): Comment
            params (dict, optional): Any WS Parameter relevant to the object and its value in form of a dictionary
            model (RSTAB Class, optional): Model to be edited
        """

        # Client model | Member Nonlinearity
        clientObject = model.clientModel.factory.create('ns0:member_nonlinearity')

        # Clears object atributes | Sets all atributes to None
        clearAttributes(clientObject)

        # Member Nonlinearity No.
        clientObject.no = no

        # Assigned Members
        clientObject.assigned_to = ConvertToDlString(members)

        # Nonlinearity Type and Parameters
        clientObject.type = nonlinearity_type.name

        if nonlinearity_type.name == "TYPE_FAILURE_IF_TENSION_WITH_SLIPPAGE" or nonlinearity_type.name == "TYPE_FAILURE_IF_COMPRESSION_WITH_SLIPPAGE" or nonlinearity_type.name == "TYPE_SLIPPAGE":
            clientObject.slippage = parameters[0]

        elif nonlinearity_type.name == "TYPE_TEARING_IF_TENSION" or nonlinearity_type.name == "TYPE_YIELDING_IF_TENSION":
            clientObject.tension_force = parameters[0]

        elif nonlinearity_type.name == "TYPE_TEARING_IF_COMPRESSION" or nonlinearity_type.name == "TYPE_YIELDING_IF_COMPRESSION":
            clientObject.compression_force = parameters[0]

        elif nonlinearity_type.name == "TYPE_TEARING" or nonlinearity_type.name == "TYPE_YIELDING":
            clientObject.compression_force = parameters[0]
            clientObject.tension_force = parameters[1]

        # Comment
        clientObject.comment = comment

        # Adding optional parameters via dictionary
        if params:
            for key in params:
                clientObject[key] = params[key]

        # Delete None attributes for improved performance
        deleteEmptyAttributes(clientObject)

        # Add Member Nonlinearity to client model
        model.clientModel.service.set_member_nonlinearity(clientObject)



#######################################################################
# THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR
# IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
# DISCLAIMED. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT,
# INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
# (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
# SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
# HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
# STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING
# IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
# POSSIBILITY OF SUCH DAMAGE.
#
# Curtesy of https://github.com/armooo/suds_requests
# author: Jason Michalski
# email: armooo@armooo.net
# License: MIT
#######################################################################

import functools
import requests
import suds.transport as transport
import traceback
from six import BytesIO

def handle_errors(f):
    @functools.wraps(f)
    def wrapper(*args, **kwargs):
        try:
            return f(*args, **kwargs)
        except requests.HTTPError as e:
            buf = BytesIO(e.response.content)
            raise transport.TransportError(
                'Error in requests\n' + traceback.format_exc(),
                e.response.status_code,
                buf,
            )
        except requests.RequestException:
            buf = BytesIO(traceback.format_exc().encode('utf-8'))
            raise transport.TransportError(
                'Error in requests\n' + traceback.format_exc(),
                000,
                buf,
            )
    return wrapper


class RequestsTransport(transport.Transport):
    def __init__(self, session=None, api_key=None, verify=True):
        transport.Transport.__init__(self)
        self._session = session or requests.Session()
        self.api_key = api_key
        self.verify = verify

    @handle_errors
    def open(self, request):
        headers = {'Authorization': f'Bearer {self.api_key}'} if self.api_key else {}
        resp = self._session.get(request.url, headers=headers, verify=self.verify)
        resp.raise_for_status()
        return BytesIO(resp.content)

    @handle_errors
    def send(self, request):
        headers = request.headers
        if self.api_key:
            headers['Authorization'] = f'Bearer {self.api_key}'
        resp = self._session.post(
            request.url,
            data=request.message,
            headers=headers,
            verify=self.verify,
        )
        if resp.headers.get('content-type') not in ('text/xml','application/soap+xml'):
            resp.raise_for_status()
        return transport.Reply(
            resp.status_code,
            resp.headers,
            resp.content,
        )



from RSTAB.initModel import Model, clearAttributes, deleteEmptyAttributes, ConvertToDlString
from RSTAB.enums import *

class ConcreteEffectiveLength():
    def __init__(self,
                no: int = 1,
                name: str = "EL 1",
                members_no: str = "1",
                member_sets_no: str = "1",
                flexural_buckling_about_y: list = [True, ConcreteEffectiveLengthAxisY.STRUCTURE_TYPE_UNBRACED],
                flexural_buckling_about_z: list = [True, ConcreteEffectiveLengthsAxisZ.STRUCTURE_TYPE_UNBRACED],
                nodal_supports: list = [[EffectiveLengthSupportType.SUPPORT_TYPE_FIXED_ALL, True,
                                         EffectiveLengthEccentricityType.ECCENTRICITY_TYPE_NONE,
                                         SupportStatus.SUPPORT_STATUS_YES,
                                         RestraintTypeAboutX.SUPPORT_STATUS_NO,
                                         RestraintTypeAboutZ.SUPPORT_STATUS_NO,
                                         RestraintTypeWarping.SUPPORT_STATUS_NO, ""]],
                factors: list = [[1, 1]],
                comment: str = '',
                params: dict = None,
                model = Model):
        """
        Args:
            no (int): Concrete Effective Length Tag
            name (str): User Defined Name
            members_no (str): Assigned Members
            member_sets_no (str): Assigned Member Sets
            flexural_buckling_about_y (list): Flexural Buckling About Y Option
            flexural_buckling_about_z (list): Flexural Buckling About Z Option
            nodal_supports (list of lists): Nodal Support Table
                nodal_supports = [[support_type, support_in_z, eccentricity_type,
                                   support_in_y_type, restraint_about_x_type, restraint_about_z_type,
                                   restraint_warping_type, nodes], ...]
            factors (list of lists): Factors Table
                factors = [[flexural_buckling_y, flexural_buckling_z]]
            comment (str, optional): Comments
            params (dict, optional): Any WS Parameter relevant to the object and its value in form of a dictionary
            model (RSTAB Class, optional): Model to be edited
        """

        # Client model | Concrete Durabilities
        clientObject = model.clientModel.factory.create('ns0:concrete_effective_lengths')

        # Clears object atributes | Sets all atributes to None
        clearAttributes(clientObject)

        # Concrete Durability No.
        clientObject.no = no

        # User Defined Name
        if name:
            clientObject.user_defined_name_enabled = True
            clientObject.name = name

        # Assigned Members
        clientObject.members = ConvertToDlString(members_no)

        # Assigned Member Sets
        clientObject.member_sets = ConvertToDlString(member_sets_no)

        # Flexural Buckling
        clientObject.flexural_buckling_about_y = flexural_buckling_about_y[0]
        clientObject.structure_type_about_axis_y = flexural_buckling_about_y[1].name
        clientObject.flexural_buckling_about_z = flexural_buckling_about_z[0]
        clientObject.structure_type_about_axis_z = flexural_buckling_about_z[1].name

        # Factors
        clientObject.factors = model.clientModel.factory.create('ns0:array_of_concrete_effective_lengths_factors')
        for i in range(len(factors)):
            mlvlp = model.clientModel.factory.create('ns0:concrete_effective_lengths_factors_row')
            mlvlp.no = i+1
            mlvlp.row = model.clientModel.factory.create('ns0:concrete_effective_lengths_factors')
            clearAttributes(mlvlp.row)
            mlvlp.row.flexural_buckling_y = factors[i][0]
            mlvlp.row.flexural_buckling_z = factors[i][1]

            clientObject.factors.concrete_effective_lengths_factors.append(mlvlp)

        # Nodal Supports
        clientObject.nodal_supports = model.clientModel.factory.create('ns0:array_of_concrete_effective_lengths_nodal_supports')
        for i in range(len(nodal_supports)):
            mlvlp = model.clientModel.factory.create('ns0:concrete_effective_lengths_nodal_supports_row')
            mlvlp.no = i+1
            mlvlp.row = model.clientModel.factory.create('ns0:concrete_effective_lengths_nodal_supports')
            clearAttributes(mlvlp.row)
            mlvlp.row.support_type = nodal_supports[i][0].name
            mlvlp.row.support_in_z = nodal_supports[i][1]
            mlvlp.row.eccentricity_type = nodal_supports[i][2].name
            mlvlp.row.support_in_y_type = nodal_supports[i][3].name
            mlvlp.row.restraint_about_x_type = nodal_supports[i][4].name
            mlvlp.row.restraint_about_z_type = nodal_supports[i][5].name
            mlvlp.row.restraint_warping_type = nodal_supports[i][6].name
            mlvlp.row.nodes = ConvertToDlString(nodal_supports[i][7])

            clientObject.nodal_supports.concrete_effective_lengths_nodal_supports.append(mlvlp)

        # Comment
        clientObject.comment = comment

        # Adding optional parameters via dictionary
        if params:
            for key in params:
                clientObject[key] = params[key]

        # Delete None attributes for improved performance
        deleteEmptyAttributes(clientObject)

        # Add Global Parameter to client model
        model.clientModel.service.set_concrete_effective_lengths(clientObject)



from RSTAB.initModel import Model, clearAttributes, deleteEmptyAttributes, ConvertToDlString
from RSTAB.enums import NodalLoadDirection, NodalLoadType, NodalLoadSpecificDirectionType

class NodalLoad():

    def __init__(self,
                 no: int = 1,
                 load_case_no: int = 1,
                 nodes_no: str= '1',
                 load_direction= NodalLoadDirection.LOAD_DIRECTION_GLOBAL_Z_OR_USER_DEFINED_W,
                 magnitude: float = 0.0,
                 comment: str= '',
                 params: dict= None,
                 model = Model):
        """
        Args:
            no (int): Load Tag
            load_case_no (int): Assigned Load Case
            nodes_no (str): Assigned Nodes
            load_direction (enum): Load Direction Enumeration
            magnitude (float): Force Magnitude
            comment (str, optional): Comments
            params (dict, optional): Any WS Parameter relevant to the object and its value in form of a dictionary
            model (RSTAB Class, optional): Model to be edited
        """
        # Client model | Nodal Force
        clientObject = model.clientModel.factory.create('ns0:nodal_load')

        # Clears object attributes | Sets all attributes to None
        clearAttributes(clientObject)

        # Nodal Force No.
        clientObject.no = no

        # Load Case No.
        clientObject.load_case = load_case_no

        # Nodes No. (e.g. '5 6 7 12')
        clientObject.nodes = ConvertToDlString(nodes_no)

        # Force Direction
        clientObject.load_direction = load_direction.name

        # Load Type
        load_type = NodalLoadType.LOAD_TYPE_FORCE
        clientObject.load_type = load_type.name

        # Magnitude
        clientObject.force_magnitude = magnitude

        # Comment
        clientObject.comment = comment

        # Adding optional parameters via dictionary
        if params:
            for key in params:
                clientObject[key] = params[key]

        # Delete None attributes for improved performance
        deleteEmptyAttributes(clientObject)

        # Add Nodal Force to client model
        model.clientModel.service.set_nodal_load(load_case_no, clientObject)

    @staticmethod
    def Force(
              no: int= 1,
              load_case_no: int = 1,
              nodes_no: str= '1',
              load_direction= NodalLoadDirection.LOAD_DIRECTION_GLOBAL_Z_OR_USER_DEFINED_W,
              magnitude: float= 0.0,
              force_eccentricity: bool = False,
              specific_direction: bool = False,
              shifted_display: bool = False,
              comment: str= '',
              params: dict = None,
              model = Model):
        """
        Args:
            no (int): Load Tag
            load_case_no (int): Assigned Load Case
            nodes_no (str): Assigned Nodes
            load_direction (enum): Load Direction Enumeration
            magnitude (float): Force Magnitude
            force_eccentricity (bool): Enable/Disable Force Eccentricity Option
            specific_direction (bool): Enable/Disable Specific Direction Option
            shifted_display (bool): Enable/Disable Shifted Display Option
            comment (str, optional): Comments
            params (dict, optional):
                For specific_direction type DIRECTION_TYPE_ROTATED_VIA_3_ANGLES;
                    params={'specific_direction' : [NodalLoadSpecificDirectionType.DIRECTION_TYPE_ROTATED_VIA_3_ANGLES, NodalLoadAxesSequence, angle_1, angle_2, angle_3, angle_x, angle_y, angle_z]}
                For specific_direction type DIRECTION_TYPE_DIRECTED_TO_NODE;
                    params={'specific_direction' : [NodalLoadSpecificDirectionType.DIRECTION_TYPE_DIRECTED_TO_NODE, nodes_no]}
                For specific_direction type DIRECTION_TYPE_PARALLEL_TO_TWO_NODES;
                    params={'specific_direction' : [NodalLoadSpecificDirectionType.DIRECTION_TYPE_PARALLEL_TO_TWO_NODES, nodes_no]}
                For specific_direction type DIRECTION_TYPE_PARALLEL_TO_CS_OF_MEMBER;
                    params={'specific_direction' : [NodalLoadSpecificDirectionType.DIRECTION_TYPE_PARALLEL_TO_CS_OF_MEMBER, member_no]}
                For force_eccentricity;
                    params={'force_eccentricity' : [ex, ey, ez]}
                For shifted_display;
                    params={'shifted_display' : [offset_x, offset_y, offset_y, distance]}
            model (RSTAB Class, optional): Model to be edited
        """
        # Client model | Nodal Force
        clientObject = model.clientModel.factory.create('ns0:nodal_load')

        # Clears object attributes | Sets all attributes to None
        clearAttributes(clientObject)

        # Nodal Force No.
        clientObject.no = no

        # Load Case No.
        clientObject.load_case = load_case_no

        # Nodes No. (e.g. '5 6 7 12')
        clientObject.nodes = ConvertToDlString(nodes_no)

        # Force Direction
        clientObject.load_direction = load_direction.name

        # Load Type
        load_type = NodalLoadType.LOAD_TYPE_FORCE
        clientObject.load_type = load_type.name

        # Force Magnitude
        clientObject.force_magnitude = magnitude

        # Option Check
        if force_eccentricity and shifted_display:
            raise ValueError("Only one of force_eccentiricity and shifted_display could be TRUE")

        # Specific Direction
        if specific_direction:

            if 'specific_direction' not in list(params.keys()):
                raise ValueError("Required key is missing")

            params_s = params['specific_direction']

            clientObject.has_specific_direction = specific_direction
            clientObject.specific_direction_type = params_s[0].name

            if params_s[0] == NodalLoadSpecificDirectionType.DIRECTION_TYPE_ROTATED_VIA_3_ANGLES:
                clientObject.axes_sequence = params_s[1].name
                clientObject.rotated_about_angle_1 = params_s[2]
                clientObject.rotated_about_angle_2 = params_s[3]
                clientObject.rotated_about_angle_3 = params_s[4]
                clientObject.rotated_about_angle_x = params_s[5]
                clientObject.rotated_about_angle_y = params_s[6]
                clientObject.rotated_about_angle_z = params_s[7]

            elif params_s[0] == NodalLoadSpecificDirectionType.DIRECTION_TYPE_DIRECTED_TO_NODE:
                clientObject.directed_to_node_direction_node = params_s[1]

            elif params_s[0] == NodalLoadSpecificDirectionType.DIRECTION_TYPE_PARALLEL_TO_TWO_NODES:
                clientObject.parallel_to_two_nodes_first_node = params_s[1]
                clientObject.parallel_to_two_nodes_second_node = params_s[2]

            elif params_s[0] == NodalLoadSpecificDirectionType.DIRECTION_TYPE_PARALLEL_TO_CS_OF_MEMBER:
                clientObject.parallel_to_member = params_s[1]

        # Force Eccentiricity
        if force_eccentricity:

            if 'force_eccentricity' not in list(params.keys()):
                raise ValueError("Required key is missing")

            params_e = params['force_eccentricity']

            clientObject.has_force_eccentricity = True

            clientObject.force_eccentricity_x = params_e[0]
            clientObject.force_eccentricity_y = params_e[1]
            clientObject.force_eccentricity_z = params_e[2]

        # Shifted Display
        if shifted_display:

            if 'shifted_display' not in list(params.keys()):
                raise ValueError("Required key is missing")

            params_d = params['shifted_display']

            clientObject.has_shifted_display = shifted_display

            clientObject.offset_x = params_d[0]
            clientObject.offset_y = params_d[1]
            clientObject.offset_z = params_d[2]
            clientObject.size_or_distance = params_d[3]

        # Comment
        clientObject.comment = comment

        # Adding optional parameters via dictionary
        if 'specific_direction' or 'force_eccentricity' or 'force_eccentricity' in params.keys():
            pass
        else:
            if params:
                for key in params:
                    clientObject[key] = params[key]

        # Delete None attributes for improved performance
        deleteEmptyAttributes(clientObject)

        # Add Nodal Force to client model
        model.clientModel.service.set_nodal_load(load_case_no, clientObject)

    @staticmethod
    def Moment(
              no: int= 1,
              load_case_no: int= 1,
              nodes_no: str= '1',
              load_direction= NodalLoadDirection.LOAD_DIRECTION_GLOBAL_Z_OR_USER_DEFINED_W,
              moment_magnitude: float= 0.0,
              specific_direction: bool= False,
              shifted_display: bool= False,
              comment: str = '',
              params: dict = None,
              model = Model):
        """
        Args:
            no (int): Load Tag
            load_case_no (int): Assigned Load Case
            nodes_no (str): Assigned Nodes
            load_direction (enum): Load Direction Enumeration
            moment_magnitude (float): Moment Magnitude
            specific_direction (bool): Enable/Disable Specific Direction Option
            shifted_display (bool): Enable/Disable Shifted Display Option
            comment (str, optional): Comments
            params (dict, optional):
                For specific_direction type DIRECTION_TYPE_ROTATED_VIA_3_ANGLES;
                    params={'specific_direction' : [NodalLoadSpecificDirectionType.DIRECTION_TYPE_ROTATED_VIA_3_ANGLES, NodalLoadAxesSequence, angle_1, angle_2, angle_3, angle_x, angle_y, angle_z]}
                For specific_direction type DIRECTION_TYPE_DIRECTED_TO_NODE;
                    params={'specific_direction' : [NodalLoadSpecificDirectionType.DIRECTION_TYPE_DIRECTED_TO_NODE, nodes_no]}
                For specific_direction type DIRECTION_TYPE_PARALLEL_TO_TWO_NODES;
                    params={'specific_direction' : [NodalLoadSpecificDirectionType.DIRECTION_TYPE_PARALLEL_TO_TWO_NODES, nodes_no]}
                For specific_direction type DIRECTION_TYPE_PARALLEL_TO_CS_OF_MEMBER;
                    params={'specific_direction' : [NodalLoadSpecificDirectionType.DIRECTION_TYPE_PARALLEL_TO_CS_OF_MEMBER, member_no]}
                For shifted_display;
                    params={'shifted_display' : [offset_x, offset_y, offset_y, distance]}
            model (RSTAB Class, optional): Model to be edited
        """
        # Client model | Nodal Force
        clientObject = model.clientModel.factory.create('ns0:nodal_load')

        # Clears object attributes | Sets all attributes to None
        clearAttributes(clientObject)

        # Nodal Force No.
        clientObject.no = no

        # Load Case No.
        clientObject.load_case = load_case_no

        # Nodes No. (e.g. '5 6 7 12')
        clientObject.nodes = ConvertToDlString(nodes_no)

        # Force Direction
        clientObject.load_direction = load_direction.name

        # Load Type
        load_type = NodalLoadType.LOAD_TYPE_MOMENT
        clientObject.load_type = load_type.name

        # Force Magnitude
        clientObject.moment_magnitude = moment_magnitude

        # Specific Direction
        if specific_direction:

            if 'specific_direction' not in list(params.keys()):
                raise ValueError("Required key is missing")

            params_s = params['specific_direction']

            clientObject.has_specific_direction = specific_direction
            clientObject.specific_direction_type = params_s[0].name

            if params_s[0] == NodalLoadSpecificDirectionType.DIRECTION_TYPE_ROTATED_VIA_3_ANGLES:
                clientObject.axes_sequence = params_s[1].name
                clientObject.rotated_about_angle_1 = params_s[2]
                clientObject.rotated_about_angle_2 = params_s[3]
                clientObject.rotated_about_angle_3 = params_s[4]
                clientObject.rotated_about_angle_x = params_s[5]
                clientObject.rotated_about_angle_y = params_s[6]
                clientObject.rotated_about_angle_z = params_s[7]

            elif params_s[0] == NodalLoadSpecificDirectionType.DIRECTION_TYPE_DIRECTED_TO_NODE:
                clientObject.directed_to_node_direction_node = params_s[1]

            elif params_s[0] == NodalLoadSpecificDirectionType.DIRECTION_TYPE_PARALLEL_TO_TWO_NODES:
                clientObject.parallel_to_two_nodes_first_node = params_s[1]
                clientObject.parallel_to_two_nodes_second_node = params_s[2]

            elif params_s[0] == NodalLoadSpecificDirectionType.DIRECTION_TYPE_PARALLEL_TO_CS_OF_MEMBER:
                clientObject.parallel_to_member = params_s[1]

        # Shifted Display
        if shifted_display:

            if 'shifted_display' not in list(params.keys()):
                raise ValueError("Required key is missing")

            params_d = params['shifted_display']

            clientObject.has_shifted_display = shifted_display

            clientObject.offset_x = params_d[0]
            clientObject.offset_y = params_d[1]
            clientObject.offset_z = params_d[2]
            clientObject.size_or_distance = params_d[3]

        # Comment
        clientObject.comment = comment

        # Adding optional parameters via dictionary
        if 'specific_direction' or 'force_eccentricity' in params.keys():
            pass
        else:
            if params:
                for key in params:
                    clientObject[key] = params[key]

        # Delete None attributes for improved performance
        deleteEmptyAttributes(clientObject)

        # Add Nodal Force to client model
        model.clientModel.service.set_nodal_load(load_case_no, clientObject)

    @staticmethod
    def Components(
              no: int= 1,
              load_case_no: int= 1,
              nodes_no: str= '1',
              components: list = None,
              specific_direction: bool= False,
              force_eccentricity: bool= False,
              shifted_display: bool= False,
              comment: str= '',
              params: dict = None,
              model = Model):
        """
        Args:
            no (int): Load Tag
            load_case_no (int): Assigned Load Case
            nodes_no (str): Assigned Nodes
            components (list): Component Magnitude List
            specific_direction (bool): Enable/Disable Specific Direction Option
            force_eccentricity(bool): Enable/Disable Force Direction Option
            shifted_display(bool): Enable/Disable Shifted Display Option
            comment (str, optional): Comments
            params (dict, optional):
                For specific_direction type DIRECTION_TYPE_ROTATED_VIA_3_ANGLES;
                    params={'specific_direction' : [NodalLoadSpecificDirectionType.DIRECTION_TYPE_ROTATED_VIA_3_ANGLES, NodalLoadAxesSequence, angle_1, angle_2, angle_3, angle_x, angle_y, angle_z]}
                For specific_direction type DIRECTION_TYPE_PARALLEL_TO_CS_OF_MEMBER;
                    params={'specific_direction' : [NodalLoadSpecificDirectionType.DIRECTION_TYPE_PARALLEL_TO_CS_OF_MEMBER, member_no]}
                For force_eccentricity;
                    params={'force_eccentricity' : [ex, ey, ez]}

                For shifted_display;
                    params={'shifted_display' : [offset_x, offset_y, offset_y, distance]}
            model (RSTAB Class, optional): Model to be edited
        """
        # Client model | Nodal Force
        clientObject = model.clientModel.factory.create('ns0:nodal_load')

        # Clears object attributes | Sets all attributes to None
        clearAttributes(clientObject)

        # Nodal Force No.
        clientObject.no = no

        # Load Case No.
        clientObject.load_case = load_case_no

        # Nodes No. (e.g. '5 6 7 12')
        clientObject.nodes = ConvertToDlString(nodes_no)

        # Load Type
        load_type = NodalLoadType.LOAD_TYPE_COMPONENTS
        clientObject.load_type = load_type.name

        # Load Magnitudes
        if len(components) == 6:
            clientObject.components_force_x = components[0]
            clientObject.components_force_y = components[1]
            clientObject.components_force_z = components[2]

            clientObject.components_moment_x = components[3]
            clientObject.components_moment_y = components[4]
            clientObject.components_moment_z = components[5]
        else:
            raise ValueError("WARNING: The components must contain 6 elements. Kindly check list inputs for completeness and correctness.")

        # Option Check
        if force_eccentricity and shifted_display:
            raise ValueError("WARNING: Only one of force_eccentiricity and shifted_display could be TRUE")

        # Specific Direction
        if specific_direction:

            if 'specific_direction' not in list(params.keys()):
                raise ValueError("Required key is missing")

            params_s = params['specific_direction']

            clientObject.has_specific_direction = specific_direction
            clientObject.specific_direction_type = params_s[0].name

            if params_s[0] == NodalLoadSpecificDirectionType.DIRECTION_TYPE_ROTATED_VIA_3_ANGLES:
                clientObject.axes_sequence = params_s[1].name
                clientObject.rotated_about_angle_1 = params_s[2]
                clientObject.rotated_about_angle_2 = params_s[3]
                clientObject.rotated_about_angle_3 = params_s[4]
                clientObject.rotated_about_angle_x = params_s[5]
                clientObject.rotated_about_angle_y = params_s[6]
                clientObject.rotated_about_angle_z = params_s[7]

            elif params_s[0] == NodalLoadSpecificDirectionType.DIRECTION_TYPE_PARALLEL_TO_CS_OF_MEMBER:
                clientObject.parallel_to_member = params_s[1]

        # Force Eccentiricity
        if force_eccentricity:

            if 'force_eccentricity' not in list(params.keys()):
                raise ValueError("Required key is missing")

            params_e = params['force_eccentricity']

            clientObject.has_force_eccentricity = force_eccentricity

            clientObject.force_eccentricity_x = params_e[0]
            clientObject.force_eccentricity_y = params_e[1]
            clientObject.force_eccentricity_z = params_e[2]

        # Shifted Display
        if shifted_display:

            if 'shifted_display' not in list(params.keys()):
                raise ValueError("Required key is missing")

            params_d = params['shifted_display']

            clientObject.has_shifted_display = shifted_display

            clientObject.offset_x = params_d[0]
            clientObject.offset_y = params_d[1]
            clientObject.offset_z = params_d[2]
            clientObject.size_or_distance = params_d[3]

        # Comment
        clientObject.comment = comment

        # Adding optional parameters via dictionary
        if 'specific_direction' or 'force_eccentricity' or 'force_eccentricity' in params.keys():
            pass
        else:
            if params:
                for key in params:
                    clientObject[key] = params[key]

        # Delete None attributes for improved performance
        deleteEmptyAttributes(clientObject)

        # Add Nodal Force to client model
        model.clientModel.service.set_nodal_load(load_case_no, clientObject)

    @staticmethod
    def Mass(
              no: int = 1,
              load_case_no: int = 1,
              nodes_no: str = '1',
              individual_mass_components : bool = False,
              mass: list = None,
              comment: str = '',
              params: dict = None,
              model = Model):
        """
        Args:
            no (int): Load Tag
            load_case_no (int): Load Case Number
            nodes_no (str): Assigned Nodes
            individual_mass_components (bool): Enable/Disable Mass Component Option
            mass (list):
                if individual_mass_components == False:
                    mass = [M]
                elif individual_mass_components == True:
                    mass = [Mx, My, Mz, Ix, Iy, Iz]
            comment (str, optional): Comments
            params (dict, optional): Any WS Parameter relevant to the object and its value in form of a dictionary
            model (RSTAB Class, optional): Model to be edited
        """
        # Client model | Nodal Force
        clientObject = model.clientModel.factory.create('ns0:nodal_load')

        # Clears object attributes | Sets all attributes to None
        clearAttributes(clientObject)

        # Nodal Force No.
        clientObject.no = no

        # Load Case No.
        clientObject.load_case = load_case_no

        # Nodes No. (e.g. '5 6 7 12')
        clientObject.nodes = ConvertToDlString(nodes_no)

        # Load Type
        load_type = NodalLoadType.LOAD_TYPE_MASS
        clientObject.load_type = load_type.name

        # Magnitude
        if individual_mass_components:

            clientObject.individual_mass_components = individual_mass_components
            clientObject.mass_x = mass[0]
            clientObject.mass_y = mass[1]
            clientObject.mass_z = mass[2]

            clientObject.mass_moment_of_inertia_x = mass[3]
            clientObject.mass_moment_of_inertia_y = mass[4]
            clientObject.mass_moment_of_inertia_z = mass[5]

        else:
            clientObject.individual_mass_components = individual_mass_components
            clientObject.mass_global = mass[0]

        # Comment
        clientObject.comment = comment

        # Adding optional parameters via dictionary
        if params:
            for key in params:
                clientObject[key] = params[key]

        # Delete None attributes for improved performance
        deleteEmptyAttributes(clientObject)

        # Add Nodal Force to client model
        model.clientModel.service.set_nodal_load(load_case_no, clientObject)



from RSTAB.initModel import Model, clearAttributes, deleteEmptyAttributes, ConvertToDlString
from RSTAB.dataTypes import inf
from RSTAB.enums import NodalSupportType, NodalSupportNonlinearity

def setNodalSupportConditions(clientObject,
                              C_u_X: float,
                              C_u_Y: float,
                              C_u_Z: float,
                              C_phi_X: float,
                              C_phi_Y: float,
                              C_phi_Z: float):
    '''
    Sets Nodal Support Conditions

    Args:
        clientObject: Client Model Object | Nodal Support
        C_u_X,Y,Z (float): Translational Support Conditions in Respected Direction
        C_phi_X,Y,Z (float): Rotational Support Conditions about Respected Axis

    Returns:
        clientObject: Initialized Client Model Object | Nodal Support
    '''

    clientObject.spring_x = C_u_X
    clientObject.spring_y = C_u_Y
    clientObject.spring_z = C_u_Z

    clientObject.rotational_restraint_x = C_phi_X
    clientObject.rotational_restraint_y = C_phi_Y
    clientObject.rotational_restraint_z = C_phi_Z

    return clientObject

class NodalSupport():
    def __init__(self,
                 no: int = 1,
                 nodes_no: str = '1 2',
                 support = NodalSupportType.HINGED,
                 comment: str = '',
                 params: dict = None,
                 model = Model):
        """
        Nodal Support

        Args:
            no (int): Nodal Support Tag
            nodes_no (str): Assigned to Nodes
            support (enum or list): Support Definition List
            comment (str, optional): Comment
            params (dict, optional): Any WS Parameter relevant to the object and its value in form of a dictionary
            model (RSTAB Class, optional): Model to be edited

        Raises:
            ValueError: 'Support parameter can be enum or list with 6 items.'
        """

        # Client model | Nodal Support
        clientObject = model.clientModel.factory.create('ns0:nodal_support')

        # Clears object atributes | Sets all atributes to None
        clearAttributes(clientObject)

        # Nodal Support No.
        clientObject.no = no

        # Nodes No. (e.g. "5 6 7 12")
        clientObject.nodes = ConvertToDlString(nodes_no)

        # Nodal Support Conditions
        if   support == NodalSupportType.FIXED:
            # FIXED       'xxx xxx'
            clientObject = setNodalSupportConditions(clientObject, inf, inf, inf, inf, inf, inf)

        elif support == NodalSupportType.HINGED:
            # HINGED      'xxx --x'
            clientObject = setNodalSupportConditions(clientObject, inf, inf, inf, 0.0, 0.0, inf)

        elif support == NodalSupportType.ROLLER:
            # ROLLER      '--x --x'
            clientObject = setNodalSupportConditions(clientObject, 0.0, 0.0, inf, 0.0, 0.0, inf)

        elif support == NodalSupportType.ROLLER_IN_X:
            # ROLLER_IN_X '-xx --x'
            clientObject = setNodalSupportConditions(clientObject, 0.0, inf, inf, 0.0, 0.0, inf)

        elif support == NodalSupportType.ROLLER_IN_Y:
            # ROLLER_IN_Y 'x-x --x'
            clientObject = setNodalSupportConditions(clientObject, inf, 0.0, inf, 0.0, 0.0, inf)

        elif support == NodalSupportType.ROLLER_IN_Z:
            # ROLLER_IN_Z 'xx- --x'
            clientObject = setNodalSupportConditions(clientObject, inf, inf, 0.0, 0.0, 0.0, inf)

        elif support == NodalSupportType.FREE:
            # FREE '--- ---'
            clientObject = setNodalSupportConditions(clientObject, 0, 0, 0, 0, 0, 0)

        elif isinstance(support, list) and len(support) == 6:
            clientObject = setNodalSupportConditions(clientObject, support[0], support[1], support[2], support[3], support[4], support[5])

        else:
            raise ValueError('Support parameter can be enum or list with 6 items.')

        # Comment
        clientObject.comment = comment

        # Adding optional parameters via dictionary
        if params:
            for key in params:
                clientObject[key] = params[key]

        # Delete None attributes for improved performance
        deleteEmptyAttributes(clientObject)

        # Add Nodal Support to client model
        model.clientModel.service.set_nodal_support(clientObject)

    @staticmethod
    def Nonlinearity(no: int = 1,
                     nodes: str = "1",
                     coordinate_system: int = 1,
                     spring_constant: list = [inf, inf, inf, 0.0, 0.0, inf],
                     spring_x_nonlinearity: list = [NodalSupportNonlinearity.NONLINEARITY_TYPE_NONE],
                     spring_y_nonlinearity: list = [NodalSupportNonlinearity.NONLINEARITY_TYPE_NONE],
                     spring_z_nonlinearity: list = [NodalSupportNonlinearity.NONLINEARITY_TYPE_NONE],
                     rotational_x_nonlinearity: list = [NodalSupportNonlinearity.NONLINEARITY_TYPE_NONE],
                     rotational_y_nonlinearity: list = [NodalSupportNonlinearity.NONLINEARITY_TYPE_NONE],
                     rotational_z_nonlinearity: list = [NodalSupportNonlinearity.NONLINEARITY_TYPE_NONE],
                     name: str = None,
                     comment: str = '',
                     params: dict = None,
                     model = Model):

        '''
        Args:
            no (int): Nodal Support Tag
            nodes (str): Assigned Nodes
            coordinate_system (int): Assigned Coordinate System
            spring_constant (list): Spring Constant List
                spring_constant = [spring_x, spring_y, spring_z, rotational_restraint_x, rotational_restraint_y, rotational_restraint_z]
            spring_x_nonlinearity (list/list of lists): Nonlinearity Parameters for Nodal Support along X direction
            spring_y_nonlinearity (list/list of lists): Nonlinearity Parameters for Nodal Support along Y direction
            spring_z_nonlinearity (list/list of lists): Nonlinearity Parameters for Nodal Support along Z direction
                for spring_x/y/z_nonlinearity[0] == NodalSupportNonlinearity.NONLINEARITY_TYPE_PARTIAL_ACTIVITY:
                    spring_x/y/z_nonlinearity = [Nodal Support nonlinearity Partial_Activity, negative zone, positive zone]
                    for negative/positive zone[0] == SupportPartialActivityAlongType.PARTIAL_ACTIVITY_TYPE_COMPLETE:
                        negative/positive zone = [negative/positive zone type, slippage]
                    for negative/positive zone[0] == SupportPartialActivityAlongType.PARTIAL_ACTIVITY_TYPE_FIXED:
                        negative/positive zone = [negative/positive zone type, slippage, displacement]  (Note: Displacement must be greater than slippage)
                    for negative/positive zone[0] == SupportPartialActivityAlongType.PARTIAL_ACTIVITY_TYPE_FAILURE_FROM_FORCE/PARTIAL_ACTIVITY_TYPE_YIELDING_FROM_FORCE:
                        negative/positive zone = [negative/positive zone type, slippage, force]
                for spring_x/y/z_nonlinearity[0] == NodalSupportNonlinearity.NONLINEARITY_TYPE_DIAGRAM:
                    spring_x/y/z_nonlinearity = [nonlinearity_type_diagram, [symmetric(bool), diagram_type_enumeration(start), diagram_type_enumeration(end)], [[displacement, force],...]]
                for spring_x/y/z_nonlinearity[0] == NodalSupportNonlinearity.NONLINEARITY_TYPE_FRICTION_DIRECTION_1/NONLINEARITY_TYPE_FRICTION_DIRECTION_2/NONLINEARITY_TYPE_FRICTION_DIRECTION_1_2:
                    spring_x/y/z_nonlinearity = [nonlinearity_type_friction, friction_coefficient]
                for spring_x/y/z_nonlinearity[0] == NodalSupportNonlinearity.NONLINEARITY_TYPE_FRICTION_DIRECTION_1_PLUS_2:
                    spring_x/y/z_nonlinearity = [nonlinearity_type_friction, friction_coefficient_1, friction_coefficient_2]
            rotational_x_nonlinearity (list of lists): Nonlinearity Parameters for Nodal Support around X direction
            rotational_y_nonlinearity (list of lists): Nonlinearity Parameters for Nodal Support around Y direction
            rotational_z_nonlinearity (list of lists): Nonlinearity Parameters for Nodal Support around Z direction
                for rotational_x/y/z_nonlinearity[0] == NodalSupportNonlinearity.NONLINEARITY_TYPE_PARTIAL_ACTIVITY:
                    rotational_x/y/z_nonlinearity = [Nodal Support nonlinearity Partial_Activity, negative zone, positive zone]
                    for negative/positive zone[0] == SupportPartialActivityAroundType.PARTIAL_ACTIVITY_TYPE_COMPLETE:
                        negative/positive zone = [negative/positive zone type, slippage]
                    for negative/positive zone[0] == SupportPartialActivityAroundType.PARTIAL_ACTIVITY_TYPE_FIXED:
                        negative/positive zone = [negative/positive zone type, slippage, rotation]
                    for negative/positive zone[0] == SupportPartialActivityAroundType.PARTIAL_ACTIVITY_TYPE_FAILURE_FROM_MOMENT/PARTIAL_ACTIVITY_TYPE_YIELDING_FROM_MOMENT:
                        negative/positive zone = [negative/positive zone type, slippage, moment]
                for rotational_x/y/z_nonlinearity[0] == NodalSupportNonlinearity.NONLINEARITY_TYPE_DIAGRAM:
                    rotational_x/y/z_nonlinearity = [nonlinearity_type_diagram, [symmetric(bool), diagram_type_enumeration(start), diagram_type_enumeration(end)], [[rotation, moment],...]]
                for rotational_x/y/z_nonlinearity[0] == NodalSupportNonlinearity.NONLINEARITY_TYPE_STIFFNESS_DIAGRAM:
                    rotational_x/y/z_nonlinearity = [nonlinearity_type_stiffness_diagram, [stiffness_diagram_around_depends_on, symmetric(bool), diagram_type_enumeration(start), diagram_type_enumeration(end)], [[force, spring],...]]
                    (note: for stiffness_diagram_around_depends_on == SupportStiffnessDiagramDependOn.STIFFNESS_DIAGRAM_DEPENDS_ON_P: symmetric must be True.)
            name (str, optional): User Defined Nodal Support Name
            comment (str, optional): Comment
            params (dict, optional): Any WS Parameter relevant to the object and its value in form of a dictionary
            model (RSTAB Class, optional): Model to be edited
        '''

        # Client model | Nodal Support
        clientObject = model.clientModel.factory.create('ns0:nodal_support')

        # Clears object atributes | Sets all atributes to None
        clearAttributes(clientObject)

        # Nodal Support No.
        clientObject.no = no

        # Assign Node Number
        clientObject.nodes = ConvertToDlString(nodes)

        # Coordinate System
        clientObject.coordinate_system = coordinate_system

        # Nodal Support Spring Constants
        clientObject.spring_x = spring_constant[0]
        clientObject.spring_y = spring_constant[1]
        clientObject.spring_z = spring_constant[2]

        clientObject.rotational_restraint_x = spring_constant[3]
        clientObject.rotational_restraint_y = spring_constant[4]
        clientObject.rotational_restraint_z = spring_constant[5]

        # Nodal Support Nonlinearity Type
        clientObject.spring_x_nonlinearity = spring_x_nonlinearity[0].name
        clientObject.spring_y_nonlinearity = spring_y_nonlinearity[0].name
        clientObject.spring_z_nonlinearity = spring_z_nonlinearity[0].name
        clientObject.rotational_restraint_x_nonlinearity = rotational_x_nonlinearity[0].name
        clientObject.rotational_restraint_y_nonlinearity = rotational_y_nonlinearity[0].name
        clientObject.rotational_restraint_z_nonlinearity = rotational_z_nonlinearity[0].name

        # Nodal Support Nonlinearity Parameters for Partial Activity
        # For spring_x_nonlinearity
        if spring_x_nonlinearity[0].name == "NONLINEARITY_TYPE_PARTIAL_ACTIVITY":

            # Negative Zone
            clientObject.partial_activity_along_x_negative_type = spring_x_nonlinearity[1][0].name

            if spring_x_nonlinearity[1][0].name == 'PARTIAL_ACTIVITY_TYPE_COMPLETE':
                clientObject.partial_activity_along_x_negative_slippage = spring_x_nonlinearity[1][1]

            elif spring_x_nonlinearity[1][0].name == 'PARTIAL_ACTIVITY_TYPE_FIXED':
                clientObject.partial_activity_along_x_negative_slippage = spring_x_nonlinearity[1][1]
                clientObject.partial_activity_along_x_negative_displacement = spring_x_nonlinearity[1][2]

            elif spring_x_nonlinearity[1][0].name == 'PARTIAL_ACTIVITY_TYPE_FAILURE_FROM_FORCE' or \
                spring_x_nonlinearity[1][0].name == 'PARTIAL_ACTIVITY_TYPE_YIELDING_FROM_FORCE':
                clientObject.partial_activity_along_x_negative_slippage = spring_x_nonlinearity[1][1]
                clientObject.partial_activity_along_x_negative_force = spring_x_nonlinearity[1][2]

            # Positive Zone
            clientObject.partial_activity_along_x_positive_type = spring_x_nonlinearity[2][0].name

            if spring_x_nonlinearity[2][0].name == 'PARTIAL_ACTIVITY_TYPE_COMPLETE':
                clientObject.partial_activity_along_x_positive_slippage = spring_x_nonlinearity[2][1]

            elif spring_x_nonlinearity[2][0].name == 'PARTIAL_ACTIVITY_TYPE_FIXED':
                clientObject.partial_activity_along_x_positive_slippage = spring_x_nonlinearity[2][1]
                clientObject.partial_activity_along_x_positive_displacement = spring_x_nonlinearity[2][2]

            elif spring_x_nonlinearity[2][0].name == 'PARTIAL_ACTIVITY_TYPE_FAILURE_FROM_FORCE' or \
                spring_x_nonlinearity[2][0].name == 'PARTIAL_ACTIVITY_TYPE_YIELDING_FROM_FORCE':
                clientObject.partial_activity_along_x_positive_slippage = spring_x_nonlinearity[2][1]
                clientObject.partial_activity_along_x_positive_force = spring_x_nonlinearity[2][2]

        # For spring_y_nonlinearity
        if spring_y_nonlinearity[0].name == "NONLINEARITY_TYPE_PARTIAL_ACTIVITY":

            # Negative Zone
            clientObject.partial_activity_along_y_negative_type = spring_y_nonlinearity[1][0].name

            if spring_y_nonlinearity[1][0].name == 'PARTIAL_ACTIVITY_TYPE_COMPLETE':
                clientObject.partial_activity_along_y_negative_slippage = spring_y_nonlinearity[1][1]

            elif spring_y_nonlinearity[1][0].name == 'PARTIAL_ACTIVITY_TYPE_FIXED':
                clientObject.partial_activity_along_y_negative_slippage = spring_y_nonlinearity[1][1]
                clientObject.partial_activity_along_y_negative_displacement = spring_y_nonlinearity[1][2]

            elif spring_y_nonlinearity[1][0].name == 'PARTIAL_ACTIVITY_TYPE_FAILURE_FROM_FORCE' or \
                spring_y_nonlinearity[1][0].name == 'PARTIAL_ACTIVITY_TYPE_YIELDING_FROM_FORCE':
                clientObject.partial_activity_along_y_negative_slippage = spring_y_nonlinearity[1][1]
                clientObject.partial_activity_along_y_negative_force = spring_y_nonlinearity[1][2]

            # Positive Zone
            clientObject.partial_activity_along_y_positive_type = spring_y_nonlinearity[2][0].name

            if spring_y_nonlinearity[2][0].name == 'PARTIAL_ACTIVITY_TYPE_COMPLETE':
                clientObject.partial_activity_along_y_positive_slippage = spring_y_nonlinearity[2][1]

            elif spring_y_nonlinearity[2][0].name == 'PARTIAL_ACTIVITY_TYPE_FIXED':
                clientObject.partial_activity_along_y_positive_slippage = spring_y_nonlinearity[2][1]
                clientObject.partial_activity_along_y_positive_displacement = spring_y_nonlinearity[2][2]

            elif spring_y_nonlinearity[2][0].name == 'PARTIAL_ACTIVITY_TYPE_FAILURE_FROM_FORCE' or \
                spring_y_nonlinearity[2][0].name == 'PARTIAL_ACTIVITY_TYPE_YIELDING_FROM_FORCE':
                clientObject.partial_activity_along_y_positive_slippage = spring_y_nonlinearity[2][1]
                clientObject.partial_activity_along_y_positive_force = spring_y_nonlinearity[2][2]

        # For spring_z_nonlinearity
        if spring_z_nonlinearity[0].name == "NONLINEARITY_TYPE_PARTIAL_ACTIVITY":

            # Negative Zone
            clientObject.partial_activity_along_z_negative_type  = spring_z_nonlinearity[1][0].name

            if spring_z_nonlinearity[1][0].name == 'PARTIAL_ACTIVITY_TYPE_COMPLETE':
                clientObject.partial_activity_along_z_negative_slippage = spring_z_nonlinearity[1][1]

            elif spring_z_nonlinearity[1][0].name == 'PARTIAL_ACTIVITY_TYPE_FIXED':
                clientObject.partial_activity_along_z_negative_slippage = spring_z_nonlinearity[1][1]
                clientObject.partial_activity_along_z_negative_displacement = spring_z_nonlinearity[1][2]

            elif spring_z_nonlinearity[1][0].name == 'PARTIAL_ACTIVITY_TYPE_FAILURE_FROM_FORCE' or \
                spring_z_nonlinearity[1][0].name == 'PARTIAL_ACTIVITY_TYPE_YIELDING_FROM_FORCE':
                clientObject.partial_activity_along_z_negative_slippage = spring_z_nonlinearity[1][1]
                clientObject.partial_activity_along_z_negative_force = spring_z_nonlinearity[1][2]

            # Positive Zone
            clientObject.partial_activity_along_z_positive_type = spring_z_nonlinearity[2][0].name

            if spring_z_nonlinearity[2][0].name == 'PARTIAL_ACTIVITY_TYPE_COMPLETE':
                clientObject.partial_activity_along_z_positive_slippage = spring_z_nonlinearity[2][1]

            elif spring_z_nonlinearity[2][0].name == 'PARTIAL_ACTIVITY_TYPE_FIXED':
                clientObject.partial_activity_along_z_positive_slippage = spring_z_nonlinearity[2][1]
                clientObject.partial_activity_along_z_positive_displacement = spring_z_nonlinearity[2][2]

            elif spring_z_nonlinearity[2][0].name == 'PARTIAL_ACTIVITY_TYPE_FAILURE_FROM_FORCE' or \
                spring_z_nonlinearity[2][0].name == 'PARTIAL_ACTIVITY_TYPE_YIELDING_FROM_FORCE':
                clientObject.partial_activity_along_z_positive_slippage = spring_z_nonlinearity[2][1]
                clientObject.partial_activity_along_z_positive_force = spring_z_nonlinearity[2][2]

        # For rotational_x_nonlinearity
        if rotational_x_nonlinearity[0].name == "NONLINEARITY_TYPE_PARTIAL_ACTIVITY":

            # Negative Zone
            clientObject.partial_activity_around_x_negative_type  = rotational_x_nonlinearity[1][0].name

            if rotational_x_nonlinearity[1][0].name == 'PARTIAL_ACTIVITY_TYPE_COMPLETE':
                clientObject.partial_activity_around_x_negative_slippage = rotational_x_nonlinearity[1][1]

            elif rotational_x_nonlinearity[1][0].name == 'PARTIAL_ACTIVITY_TYPE_FIXED':
                clientObject.partial_activity_around_x_negative_slippage = rotational_x_nonlinearity[1][1]
                clientObject.partial_activity_around_x_negative_rotation = rotational_x_nonlinearity[1][2]

            elif rotational_x_nonlinearity[1][0].name == 'PARTIAL_ACTIVITY_TYPE_FAILURE_FROM_MOMENT' or \
                rotational_x_nonlinearity[1][0].name == 'PARTIAL_ACTIVITY_TYPE_YIELDING_FROM_MOMENT':
                clientObject.partial_activity_around_x_negative_slippage = rotational_x_nonlinearity[1][1]
                clientObject.partial_activity_around_x_negative_moment = rotational_x_nonlinearity[1][2]

            # Positive Zone
            clientObject.partial_activity_around_x_positive_type  = rotational_x_nonlinearity[2][0].name

            if rotational_x_nonlinearity[2][0].name == 'PARTIAL_ACTIVITY_TYPE_COMPLETE':
                clientObject.partial_activity_around_x_positive_slippage = rotational_x_nonlinearity[2][1]

            elif rotational_x_nonlinearity[2][0].name == 'PARTIAL_ACTIVITY_TYPE_FIXED':
                clientObject.partial_activity_around_x_positive_slippage = rotational_x_nonlinearity[2][1]
                clientObject.partial_activity_around_x_positive_rotation = rotational_x_nonlinearity[2][2]

            elif rotational_x_nonlinearity[2][0].name == 'PARTIAL_ACTIVITY_TYPE_FAILURE_FROM_MOMENT' or \
                rotational_x_nonlinearity[2][0].name == 'PARTIAL_ACTIVITY_TYPE_YIELDING_FROM_MOMENT':
                clientObject.partial_activity_around_x_positive_slippage = rotational_x_nonlinearity[2][1]
                clientObject.partial_activity_around_x_positive_moment = rotational_x_nonlinearity[2][2]

        # For rotational_y_nonlinearity
        if rotational_y_nonlinearity[0].name == "NONLINEARITY_TYPE_PARTIAL_ACTIVITY":

            # Negative Zone
            clientObject.partial_activity_around_y_negative_type  = rotational_y_nonlinearity[1][0].name

            if rotational_y_nonlinearity[1][0].name == 'PARTIAL_ACTIVITY_TYPE_COMPLETE':
                clientObject.partial_activity_around_y_negative_slippage = rotational_y_nonlinearity[1][1]

            elif rotational_y_nonlinearity[1][0].name == 'PARTIAL_ACTIVITY_TYPE_FIXED':
                clientObject.partial_activity_around_y_negative_slippage = rotational_y_nonlinearity[1][1]
                clientObject.partial_activity_around_y_negative_rotation = rotational_y_nonlinearity[1][2]

            elif rotational_y_nonlinearity[1][0].name == 'PARTIAL_ACTIVITY_TYPE_FAILURE_FROM_MOMENT' or \
                rotational_y_nonlinearity[1][0].name == 'PARTIAL_ACTIVITY_TYPE_YIELDING_FROM_MOMENT':
                clientObject.partial_activity_around_y_negative_slippage = rotational_y_nonlinearity[1][1]
                clientObject.partial_activity_around_y_negative_moment = rotational_y_nonlinearity[1][2]

            # Positive Zone
            clientObject.partial_activity_around_y_positive_type  = rotational_y_nonlinearity[2][0].name

            if rotational_y_nonlinearity[2][0].name == 'PARTIAL_ACTIVITY_TYPE_COMPLETE':
                clientObject.partial_activity_around_y_positive_slippage = rotational_y_nonlinearity[2][1]

            elif rotational_y_nonlinearity[2][0].name == 'PARTIAL_ACTIVITY_TYPE_FIXED':
                clientObject.partial_activity_around_y_positive_slippage = rotational_y_nonlinearity[2][1]
                clientObject.partial_activity_around_y_positive_rotation = rotational_y_nonlinearity[2][2]

            elif rotational_y_nonlinearity[2][0].name == 'PARTIAL_ACTIVITY_TYPE_FAILURE_FROM_MOMENT' or \
                rotational_y_nonlinearity[2][0].name == 'PARTIAL_ACTIVITY_TYPE_YIELDING_FROM_MOMENT':
                clientObject.partial_activity_around_y_positive_slippage = rotational_y_nonlinearity[2][1]
                clientObject.partial_activity_around_y_positive_moment = rotational_y_nonlinearity[2][2]

        # For rotational_z_nonlinearity
        if rotational_z_nonlinearity[0].name == "NONLINEARITY_TYPE_PARTIAL_ACTIVITY":

            # Negative Zone
            clientObject.partial_activity_around_z_negative_type  = rotational_z_nonlinearity[1][0].name

            if rotational_z_nonlinearity[1][0].name == 'PARTIAL_ACTIVITY_TYPE_COMPLETE':
                clientObject.partial_activity_around_z_negative_slippage = rotational_z_nonlinearity[1][1]

            elif rotational_z_nonlinearity[1][0].name == 'PARTIAL_ACTIVITY_TYPE_FIXED':
                clientObject.partial_activity_around_z_negative_slippage = rotational_z_nonlinearity[1][1]
                clientObject.partial_activity_around_z_negative_rotation = rotational_z_nonlinearity[1][2]

            elif rotational_z_nonlinearity[1][0].name == 'PARTIAL_ACTIVITY_TYPE_FAILURE_FROM_MOMENT' or \
                rotational_z_nonlinearity[1][0].name == 'PARTIAL_ACTIVITY_TYPE_YIELDING_FROM_MOMENT':
                clientObject.partial_activity_around_z_negative_slippage = rotational_z_nonlinearity[1][1]
                clientObject.partial_activity_around_z_negative_moment = rotational_z_nonlinearity[1][2]

            # Positive Zone
            clientObject.partial_activity_around_z_positive_type  = rotational_z_nonlinearity[2][0].name

            if rotational_z_nonlinearity[2][0].name == 'PARTIAL_ACTIVITY_TYPE_COMPLETE':
                clientObject.partial_activity_around_z_positive_slippage = rotational_z_nonlinearity[2][1]

            elif rotational_z_nonlinearity[2][0].name == 'PARTIAL_ACTIVITY_TYPE_FIXED':
                clientObject.partial_activity_around_z_positive_slippage = rotational_z_nonlinearity[2][1]
                clientObject.partial_activity_around_z_positive_rotation = rotational_z_nonlinearity[2][2]

            elif rotational_z_nonlinearity[2][0].name == 'PARTIAL_ACTIVITY_TYPE_FAILURE_FROM_MOMENT' or \
                rotational_z_nonlinearity[2][0].name == 'PARTIAL_ACTIVITY_TYPE_YIELDING_FROM_MOMENT':
                clientObject.partial_activity_around_z_positive_slippage = rotational_z_nonlinearity[2][1]
                clientObject.partial_activity_around_z_positive_moment = rotational_z_nonlinearity[2][2]

        # Nodal Support Nonlinearity Parameters for Diagram
        # For spring_x_nonlinearity
        if spring_x_nonlinearity[0].name == "NONLINEARITY_TYPE_DIAGRAM":
            clientObject.diagram_along_x_symmetric = spring_x_nonlinearity[1][0]

            if spring_x_nonlinearity[1][0]:
                clientObject.diagram_along_x_end = spring_x_nonlinearity[1][1].name
                clientObject.diagram_along_x_start = spring_x_nonlinearity[1][1].name

            else:
                clientObject.diagram_along_x_start = spring_x_nonlinearity[1][1].name
                clientObject.diagram_along_x_end = spring_x_nonlinearity[1][2].name

            clientObject.diagram_along_x_table = model.clientModel.factory.create('ns0:nodal_support.diagram_along_x_table')

            for i, j in enumerate(spring_x_nonlinearity[2]):
                nsux = model.clientModel.factory.create('ns0:nodal_support_diagram_along_x_table_row')
                nsux.no = i+1
                nsux.row = model.clientModel.factory.create('ns0:nodal_support_diagram_along_x_table')
                clearAttributes(nsux.row)
                nsux.row.displacement = spring_x_nonlinearity[2][i][0]
                nsux.row.force = spring_x_nonlinearity[2][i][1]

                clientObject.diagram_along_x_table.nodal_support_diagram_along_x_table.append(nsux)

            clientObject.diagram_along_x_is_sorted = True

        # For spring_y_nonlinearity
        if spring_y_nonlinearity[0].name == "NONLINEARITY_TYPE_DIAGRAM":
            clientObject.diagram_along_y_symmetric = spring_y_nonlinearity[1][0]

            if spring_y_nonlinearity[1][0]:
                clientObject.diagram_along_y_end = spring_y_nonlinearity[1][1].name
                clientObject.diagram_along_y_start = spring_y_nonlinearity[1][1].name

            else:
                clientObject.diagram_along_y_end = spring_y_nonlinearity[1][2].name
                clientObject.diagram_along_y_start = spring_y_nonlinearity[1][1].name

            clientObject.diagram_along_y_table = model.clientModel.factory.create('ns0:nodal_support.diagram_along_y_table')

            for i, j in enumerate(spring_y_nonlinearity[2]):
                nsuy = model.clientModel.factory.create('ns0:nodal_support_diagram_along_y_table_row')
                nsuy.no = i+1
                nsuy.row = model.clientModel.factory.create('ns0:nodal_support_diagram_along_y_table')
                clearAttributes(nsuy.row)
                nsuy.row.displacement = spring_y_nonlinearity[2][i][0]
                nsuy.row.force = spring_y_nonlinearity[2][i][1]

                clientObject.diagram_along_y_table.nodal_support_diagram_along_y_table.append(nsuy)

            clientObject.diagram_along_y_is_sorted = True

        # For spring_z_nonlinearity
        if spring_z_nonlinearity[0].name == "NONLINEARITY_TYPE_DIAGRAM":
            clientObject.diagram_along_z_symmetric = spring_z_nonlinearity[1][0]

            if spring_z_nonlinearity[1][0]:
                clientObject.diagram_along_z_end = spring_z_nonlinearity[1][1].name
                clientObject.diagram_along_z_start = spring_z_nonlinearity[1][1].name

            else:
                clientObject.diagram_along_z_start = spring_z_nonlinearity[1][1].name
                clientObject.diagram_along_z_end = spring_z_nonlinearity[1][2].name

            clientObject.diagram_along_z_table = model.clientModel.factory.create('ns0:nodal_support.diagram_along_z_table')

            for i, j in enumerate(spring_z_nonlinearity[2]):
                nsuz = model.clientModel.factory.create('ns0:nodal_support_diagram_along_z_table_row')
                nsuz.no = i+1
                nsuz.row = model.clientModel.factory.create('ns0:nodal_support_diagram_along_z_table')
                clearAttributes(nsuz.row)
                nsuz.row.displacement = spring_z_nonlinearity[2][i][0]
                nsuz.row.force = spring_z_nonlinearity[2][i][1]

                clientObject.diagram_along_z_table.nodal_support_diagram_along_z_table.append(nsuz)

            clientObject.diagram_along_z_is_sorted = True

        # For rotational_x_nonlinearity
        if rotational_x_nonlinearity[0].name == "NONLINEARITY_TYPE_DIAGRAM":
            clientObject.diagram_around_x_symmetric = rotational_x_nonlinearity[1][0]

            if rotational_x_nonlinearity[1][0]:
                clientObject.diagram_around_x_end = rotational_x_nonlinearity[1][1].name
                clientObject.diagram_around_x_start = rotational_x_nonlinearity[1][1].name

            else:
                clientObject.diagram_around_x_start = rotational_x_nonlinearity[1][1].name
                clientObject.diagram_around_x_end = rotational_x_nonlinearity[1][2].name

            clientObject.diagram_around_x_table = model.clientModel.factory.create('ns0:nodal_support.diagram_around_x_table')

            for i, j in enumerate(rotational_x_nonlinearity[2]):
                nsrx = model.clientModel.factory.create('ns0:nodal_support_diagram_around_x_table_row')
                nsrx.no = i+1
                nsrx.row = model.clientModel.factory.create('ns0:nodal_support_diagram_around_x_table')
                clearAttributes(nsrx.row)
                nsrx.row.rotation = rotational_x_nonlinearity[2][i][0]
                nsrx.row.moment = rotational_x_nonlinearity[2][i][1]

                clientObject.diagram_around_x_table.nodal_support_diagram_around_x_table.append(nsrx)

            clientObject.diagram_around_x_is_sorted = True

        # For rotational_y_nonlinearity
        if rotational_y_nonlinearity[0].name == "NONLINEARITY_TYPE_DIAGRAM":
            clientObject.diagram_around_y_symmetric = rotational_y_nonlinearity[1][0]

            if rotational_y_nonlinearity[1][0]:
                clientObject.diagram_around_y_end = rotational_y_nonlinearity[1][1].name
                clientObject.diagram_around_y_start = rotational_y_nonlinearity[1][1].name

            else:
                clientObject.diagram_around_y_start = rotational_y_nonlinearity[1][1].name
                clientObject.diagram_around_y_end = rotational_y_nonlinearity[1][2].name

            clientObject.diagram_around_y_table = model.clientModel.factory.create('ns0:nodal_support.diagram_around_y_table')

            for i, j in enumerate(rotational_y_nonlinearity[2]):
                nsry = model.clientModel.factory.create('ns0:nodal_support_diagram_around_y_table_row')
                nsry.no = i+1
                nsry.row = model.clientModel.factory.create('ns0:nodal_support_diagram_around_y_table')
                clearAttributes(nsry.row)
                nsry.row.rotation = rotational_y_nonlinearity[2][i][0]
                nsry.row.moment = rotational_y_nonlinearity[2][i][1]

                clientObject.diagram_around_y_table.nodal_support_diagram_around_y_table.append(nsry)

            clientObject.diagram_around_y_is_sorted = True

        # For rotational_z_nonlinearity
        if rotational_z_nonlinearity[0].name == "NONLINEARITY_TYPE_DIAGRAM":
            clientObject.diagram_around_z_symmetric = rotational_z_nonlinearity[1][0]

            if rotational_z_nonlinearity[1][0]:
                clientObject.diagram_around_z_end = rotational_z_nonlinearity[1][1].name
                clientObject.diagram_around_z_start = rotational_z_nonlinearity[1][1].name

            else:
                clientObject.diagram_around_z_start = rotational_z_nonlinearity[1][1].name
                clientObject.diagram_around_z_end = rotational_z_nonlinearity[1][2].name

            clientObject.diagram_around_z_table = model.clientModel.factory.create('ns0:nodal_support.diagram_around_z_table')

            for i, j in enumerate(rotational_z_nonlinearity[2]):
                nsrz = model.clientModel.factory.create('ns0:nodal_support_diagram_around_z_table_row')
                nsrz.no = i+1
                nsrz.row = model.clientModel.factory.create('ns0:nodal_support_diagram_around_z_table')
                clearAttributes(nsrz.row)
                nsrz.row.rotation = rotational_z_nonlinearity[2][i][0]
                nsrz.row.moment = rotational_z_nonlinearity[2][i][1]

                clientObject.diagram_around_z_table.nodal_support_diagram_around_z_table.append(nsrz)

            clientObject.diagram_around_z_is_sorted = True

        # Nodal Support Nonlinearity Parameters for Frictions
        # For spring_x_nonlinearity
        if spring_x_nonlinearity[0].name == "NONLINEARITY_TYPE_FRICTION_DIRECTION_1" or \
            spring_x_nonlinearity[0].name == "NONLINEARITY_TYPE_FRICTION_DIRECTION_2" or \
            spring_x_nonlinearity[0].name == "NONLINEARITY_TYPE_FRICTION_DIRECTION_1_2":
            clientObject.friction_coefficient_x = spring_x_nonlinearity[1]

        elif spring_x_nonlinearity[0].name == "NONLINEARITY_TYPE_FRICTION_DIRECTION_1_PLUS_2":
            clientObject.friction_coefficient_xy = spring_x_nonlinearity[1]
            clientObject.friction_coefficient_xz = spring_x_nonlinearity[2]

        # For spring_y_nonlinearity
        if spring_y_nonlinearity[0].name == "NONLINEARITY_TYPE_FRICTION_DIRECTION_1" or \
            spring_y_nonlinearity[0].name == "NONLINEARITY_TYPE_FRICTION_DIRECTION_2" or \
            spring_y_nonlinearity[0].name == "NONLINEARITY_TYPE_FRICTION_DIRECTION_1_2":
            clientObject.friction_coefficient_y = spring_y_nonlinearity[1]

        elif spring_y_nonlinearity[0].name == "NONLINEARITY_TYPE_FRICTION_DIRECTION_1_PLUS_2":
            clientObject.friction_coefficient_yx = spring_y_nonlinearity[1]
            clientObject.friction_coefficient_yz = spring_y_nonlinearity[2]

        # For spring_z_nonlinearity
        if spring_z_nonlinearity[0].name == "NONLINEARITY_TYPE_FRICTION_DIRECTION_1" or \
            spring_z_nonlinearity[0].name == "NONLINEARITY_TYPE_FRICTION_DIRECTION_2" or \
            spring_z_nonlinearity[0].name == "NONLINEARITY_TYPE_FRICTION_DIRECTION_1_2":
            clientObject.friction_coefficient_z = spring_z_nonlinearity[1]

        elif spring_z_nonlinearity[0].name == "NONLINEARITY_TYPE_FRICTION_DIRECTION_1_PLUS_2":
            clientObject.friction_coefficient_zx = spring_z_nonlinearity[1]
            clientObject.friction_coefficient_zy = spring_z_nonlinearity[2]

        # Nodal Support Nonlinearity Parameters for Stifness Diagram
        # For rotational_x_nonlinearity
        if rotational_x_nonlinearity[0].name == "NONLINEARITY_TYPE_STIFFNESS_DIAGRAM":

            clientObject.stiffness_diagram_around_x_depends_on = rotational_x_nonlinearity[1][0].name
            clientObject.stiffness_diagram_around_x_symmetric = rotational_x_nonlinearity[1][1]
            if rotational_x_nonlinearity[1][0].name == 'STIFFNESS_DIAGRAM_DEPENDS_ON_P':
                clientObject.stiffness_diagram_around_x_symmetric = True

            if rotational_x_nonlinearity[1][1]:
                clientObject.stiffness_diagram_around_x_end = rotational_x_nonlinearity[1][2].name
                clientObject.stiffness_diagram_around_x_start = rotational_x_nonlinearity[1][2].name

            else:
                clientObject.stiffness_diagram_around_x_start = rotational_x_nonlinearity[1][2].name
                clientObject.stiffness_diagram_around_x_end = rotational_x_nonlinearity[1][3].name

            clientObject.stiffness_diagram_around_x_table = model.clientModel.factory.create('ns0:nodal_support.stiffness_diagram_around_x_table')

            for i, j in enumerate(rotational_x_nonlinearity[2]):
                nssrx = model.clientModel.factory.create('ns0:nodal_support_stiffness_diagram_around_x_table_row')
                nssrx.no = i+1
                nssrx.row = model.clientModel.factory.create('ns0:nodal_support_stiffness_diagram_around_x_table')
                clearAttributes(nssrx.row)
                nssrx.row.force = rotational_x_nonlinearity[2][i][0]
                nssrx.row.spring = rotational_x_nonlinearity[2][i][1]

                clientObject.stiffness_diagram_around_x_table.nodal_support_stiffness_diagram_around_x_table.append(nssrx)

            clientObject.stiffness_diagram_around_x_is_sorted = True

        # For rotational_y_nonlinearity
        if rotational_y_nonlinearity[0].name == "NONLINEARITY_TYPE_STIFFNESS_DIAGRAM":

            clientObject.stiffness_diagram_around_y_depends_on = rotational_y_nonlinearity[1][0].name
            clientObject.stiffness_diagram_around_y_symmetric = rotational_y_nonlinearity[1][1]
            if rotational_y_nonlinearity[1][0].name == 'STIFFNESS_DIAGRAM_DEPENDS_ON_P':
                clientObject.stiffness_diagram_around_y_symmetric = True

            if rotational_y_nonlinearity[1][1]:
                clientObject.stiffness_diagram_around_y_end = rotational_y_nonlinearity[1][2].name
                clientObject.stiffness_diagram_around_y_start = rotational_y_nonlinearity[1][2].name

            else:
                clientObject.stiffness_diagram_around_y_start = rotational_y_nonlinearity[1][2].name
                clientObject.stiffness_diagram_around_y_end = rotational_y_nonlinearity[1][3].name

            clientObject.stiffness_diagram_around_y_table = model.clientModel.factory.create('ns0:nodal_support.stiffness_diagram_around_y_table')

            for i, j in enumerate(rotational_y_nonlinearity[2]):
                nssry = model.clientModel.factory.create('ns0:nodal_support_stiffness_diagram_around_y_table_row')
                nssry.no = i+1
                nssry.row = model.clientModel.factory.create('ns0:nodal_support_stiffness_diagram_around_y_table')
                clearAttributes(nssry.row)
                nssry.row.force = rotational_y_nonlinearity[2][i][0]
                nssry.row.spring = rotational_y_nonlinearity[2][i][1]

                clientObject.stiffness_diagram_around_y_table.nodal_support_stiffness_diagram_around_y_table.append(nssry)

            clientObject.stiffness_diagram_around_y_is_sorted = True

        # For rotational_z_nonlinearity
        if rotational_z_nonlinearity[0].name == "NONLINEARITY_TYPE_STIFFNESS_DIAGRAM":

            clientObject.stiffness_diagram_around_z_depends_on = rotational_z_nonlinearity[1][0].name
            clientObject.stiffness_diagram_around_z_symmetric = rotational_z_nonlinearity[1][1]
            if rotational_z_nonlinearity[1][0].name == 'STIFFNESS_DIAGRAM_DEPENDS_ON_P':
                clientObject.stiffness_diagram_around_z_symmetric = True

            if rotational_z_nonlinearity[1][1]:
                clientObject.stiffness_diagram_around_z_end = rotational_z_nonlinearity[1][2].name
                clientObject.stiffness_diagram_around_z_start = rotational_z_nonlinearity[1][2].name

            else:
                clientObject.stiffness_diagram_around_z_start = rotational_z_nonlinearity[1][2].name
                clientObject.stiffness_diagram_around_z_end = rotational_z_nonlinearity[1][3].name

            clientObject.stiffness_diagram_around_z_table = model.clientModel.factory.create('ns0:nodal_support.stiffness_diagram_around_z_table')

            for i, j in enumerate(rotational_z_nonlinearity[2]):
                nssrz = model.clientModel.factory.create('ns0:nodal_support_stiffness_diagram_around_z_table_row')
                nssrz.no = i+1
                nssrz.row = model.clientModel.factory.create('ns0:nodal_support_stiffness_diagram_around_z_table')
                clearAttributes(nssrz.row)
                nssrz.row.force = rotational_z_nonlinearity[2][i][0]
                nssrz.row.spring = rotational_z_nonlinearity[2][i][1]

                clientObject.stiffness_diagram_around_z_table.nodal_support_stiffness_diagram_around_z_table.append(nssrz)

            clientObject.stiffness_diagram_around_z_is_sorted = True

        # User Defined Name
        if name:
            clientObject.user_defined_name_enabled = True
            clientObject.name = name

        # Comment
        clientObject.comment = comment

        # Adding optional parameters via dictionary
        if params:
            for key in params:
                clientObject[key] = params[key]

        # Delete None attributes for improved performance
        deleteEmptyAttributes(clientObject)

        # Add Nodal Support to client model
        model.clientModel.service.set_nodal_support(clientObject)



from RSTAB.initModel import Model, clearAttributes, deleteEmptyAttributes
from RSTAB.enums import MemberRotationalRestraintContinousBeamEffect, MemberRotationalRestraintRotationalStiffness, MemberRotationalRestraintSheetingPosition, MemberRotationalRestraintType

class MemberRotationalRestraint():

    def __init__(self) -> None:
        pass

    @staticmethod
    def Continuous(
                no: int = 1,
                name: str = '',
                sheeting_material: str = 'S235',
                sheeting_name: str = 'Arval (-) 35/207 - 0.63 (b: 1) | DIN 18807 | Arval',
                position_of_sheeting = MemberRotationalRestraintSheetingPosition.SHEETING_POSITION_POSITIVE,
                continuous_beam_effect = MemberRotationalRestraintContinousBeamEffect.CONTINUOUS_BEAM_EFFECT_END_PANEL,
                section_deformation_cdb: bool = True,
                modulus_of_elasticity: float = None,
                sheeting_thickness: float = None,
                sheeting_moment_of_inertia: float = None,
                sheeting_distance_of_ribs: float = None,
                width_of_sheeting_flange: float = None,
                spring_stiffness: float = 5200.0,
                beam_spacing: float= 3.0,
                comment: str = '',
                params: dict = None,
                model = Model):

        """
        Args:
            no (int): Member Rotational Restraint Tag
            name (str): User Defined Member Rotational Restraint Name
                if name == '':
                    user_defined_name_enabled = False (Automatic Name Assignment)
                else:
                    user_defined_name_enabled = True
                    name = User Defined Name
            sheeting_material (str): Sheeting Material
            sheeting_name (str): Sheeting Name According to Library
            position_of_sheeting (enum): Position of Sheeting Enumeration
            continuous_beam_effect (enum): Continous Beam Effect Enumeration
            section_deformation_cdb (bool): Section Deformation Cdb Option
            modulus_of_elasticity (float, optional): Modulus of Elasticity
            sheeting_thickness (float, optional): Sheeting Thickness
            sheeting_moment_of_inertia (float, optional): Sheeting Moment of Inertia
            sheeting_distance_of_ribs (float, optional): Sheeting Distance of Ribs
            width_of_sheeting_flange (float, optional): Width of Sheeting Flange
            spring_stiffness (float): Spring Stiffness
            beam_spacing (float): Beam Spacing
            comment (str, optional): Comment
            params (dict, optional): Any WS Parameter relevant to the object and its value in form of a dictionary
            model (RSTAB Class, optional): Model to be edited
        """

        # Client Model | Member Rotational Restraint
        clientObject = model.clientModel.factory.create('ns0:member_rotational_restraint')

        # Clears object atributes | Sets all atributes to None
        clearAttributes(clientObject)

        # Member Rotational Restraint No.
        clientObject.no = no

        # Member Rotational Restraint User Defined Name
        if name:
            clientObject.user_defined_name_enabled = True
            clientObject.name = name

        # Member Rotational Restraint Definition Type
        clientObject.type = MemberRotationalRestraintType.TYPE_CONTINUOUS.name

        # Member Rotational Restraint Sheeting Material Name
        clientObject.material_name = sheeting_material

        # Member Rotational Restraint Sheeting Name
        clientObject.sheeting_name = sheeting_name

        # Member Rotational Restraint Position of Sheeting
        clientObject.position_of_sheeting = position_of_sheeting.name

        # Member Rotational Restraint Continuous Beam Effect
        clientObject.continuous_beam_effect = continuous_beam_effect.name

        # Member Rotational Restraint Section Deformation Cdb
        clientObject.section_deformation_cdb = section_deformation_cdb

        # Member Rotational Restraint Modulus of Elasticity
        clientObject.modulus_of_elasticity = modulus_of_elasticity

        # Member Rotational Restraint Sheeting Thickness
        clientObject.sheeting_thickness = sheeting_thickness

        # Member Rotational Restraint Sheeting Moment of Inertia
        clientObject.sheeting_moment_of_inertia = sheeting_moment_of_inertia

        # Member Rotational Restraint Sheeting Distance of Ribs
        clientObject.sheeting_distance_of_ribs = sheeting_distance_of_ribs

        # Member Rotational Restraint Width of Sheeting Flange
        clientObject.width_of_section_flange = width_of_sheeting_flange

        # Member Rotational Restraint Spring Stiffness
        clientObject.spring_stiffness = spring_stiffness

        # Member Rotational Restraint Beam Spacing
        clientObject.beam_spacing = beam_spacing

        # Comment
        clientObject.comment = comment

        # Adding optional parameters via dictionary
        if params:
            for key in params:
                clientObject[key] = params[key]

        # Delete None attributes for improved performance
        deleteEmptyAttributes(clientObject)

        # Add Member Rotational Restraint to client model
        model.clientModel.service.set_member_rotational_restraint(clientObject)

    @staticmethod
    def Discrete(
                no: int = 1,
                name: str = '',
                section_material: str = 'S235',
                section_name: str = 'CHC 60.3x3.2',
                rotational_stifness: list = [MemberRotationalRestraintRotationalStiffness.ROTATIONAL_STIFFNESS_INFINITELY],
                continous_beam_effect = MemberRotationalRestraintContinousBeamEffect.CONTINUOUS_BEAM_EFFECT_END_PANEL,
                section_deformation_cdb: bool =  True,
                modulus_of_elasticity: float = None,
                section_moment_of_inertia: float = None,
                purlin_spacing: float = 1.0,
                beam_spacing: float = 3.0,
                comment: str = '',
                params: dict = None,
                model = Model):

        """
        Args:
            no (int): Member Rotational Restraint Tag
            name (str): User Defined Member Rotational Restraint Name
                if name == '':
                    user_defined_name_enabled = False (Automatic Name Assignment)
                else:
                    user_defined_name_enabled = True
                    name = User Defined Name
            section_material (str): Section Material
            section_name (str): Section Name
            rotational_stifness (list): Rotational Stiffness
                if rotational_stiffness[0] == MemberRotationalRestraintRotationalStiffness.ROTATIONAL_STIFFNESS_INFINITELY:
                    pass
                elif rotational_stiffness[0] == MemberRotationalRestraintRotationalStiffness.ROTATIONAL_STIFFNESS_MANUALLY:
                    rotational_stiffness[1] (float): Rotational Stiffness Value
            continous_beam_effect (enum): Continous Beam Effect Enumeration
            section_deformation_cdb (bool): Section Deformation Cdb Option
            modulus_of_elasticity (float, optional): Modulus of Elasticity
            section_moment_of_inertia (float, optional): Section Moment of Inertia
            purlin_spacing (float): Purlin Spacing
            beam_spacing (float): Beam Spacing
            comment (str, optional): Comment
            params (dict, optional): Any WS Parameter relevant to the object and its value in form of a dictionary
            model (RSTAB Class, optional): Model to be edited
        """

        # Client Model | Member Rotational Restraint
        clientObject = model.clientModel.factory.create('ns0:member_rotational_restraint')

        # Clears object atributes | Sets all atributes to None
        clearAttributes(clientObject)

        # Member Rotational Restraint No.
        clientObject.no = no

        # Member Rotational Restraint User Defined Name
        if name:
            clientObject.user_defined_name_enabled = True
            clientObject.name = name

        # Member Rotational Restraint Definition Type
        clientObject.type = MemberRotationalRestraintType.TYPE_DISCRETE.name

        # Member Rotational Restraint Section Material Name
        clientObject.material_name = section_material

        # Member Rotational Restraint Section Name
        clientObject.section_name = section_name

        # Member Rotational Restraint Stiffness Cda
        clientObject.rotational_stiffness = rotational_stifness[0].name
        if rotational_stifness[0].name == MemberRotationalRestraintRotationalStiffness.ROTATIONAL_STIFFNESS_MANUALLY.name:
            clientObject.rotational_stiffness_value = rotational_stifness[1]

        # Member Rotational Restraint Continuous Beam Effect
        clientObject.continuous_beam_effect = continous_beam_effect.name

        # Member Rotational Restraint Section Deformation Cdb
        clientObject.section_deformation_cdb = section_deformation_cdb

        # Member Rotational Restraint Modulus of Elasticity
        clientObject.modulus_of_elasticity = modulus_of_elasticity

        # Member Rotational Restraint Section Moment of Inertia
        clientObject.section_moment_of_inertia = section_moment_of_inertia

        # Member Rotational Restraint Purlin Spacing
        clientObject.purlin_spacing = purlin_spacing

        # Member Rotational Restraint Beam Spacing
        clientObject.beam_spacing = beam_spacing

        # Comment
        clientObject.comment = comment

        # Adding optional parameters via dictionary
        if params:
            for key in params:
                clientObject[key] = params[key]

        # Delete None attributes for improved performance
        deleteEmptyAttributes(clientObject)

        # Add Member Rotational Restraint to client model
        model.clientModel.service.set_member_rotational_restraint(clientObject)

    @staticmethod
    def Manually(
                no: int = 1,
                name: str = '',
                rotational_spring_stiffness: float = 3000.0,
                comment: str = '',
                params: dict = None,
                model = Model):

        """
        Args:
            no (int): Member Rotational Restraint Tag
            name (str): User Defined Member Rotational Restraint Name
                if name == '':
                    user_defined_name_enabled = False (Automatic Name Assignment)
                else:
                    user_defined_name_enabled = True
                    name = User Defined Name
            rotational_spring_stiffness (float): Rotational Spring Stiffness
            comment (str, optional): Comment
            params (dict, optional): Any WS Parameter relevant to the object and its value in form of a dictionary
            model (RSTAB Class, optional): Model to be edited
        """

        # Client Model | Member Rotational Restraint
        clientObject = model.clientModel.factory.create('ns0:member_rotational_restraint')

        # Clears object atributes | Sets all atributes to None
        clearAttributes(clientObject)

        # Member Rotational Restraint No.
        clientObject.no = no

        # Member Rotational Restraint User Defined Name
        if name:
            clientObject.user_defined_name_enabled = True
            clientObject.name = name

        # Member Rotational Restraint Definition Type
        clientObject.type = MemberRotationalRestraintType.TYPE_MANUALLY.name

        # Member Rotational Restraint Total Rotational Spring Stiffness
        clientObject.total_rotational_spring_stiffness = rotational_spring_stiffness

        # Comment
        clientObject.comment = comment

        # Adding optional parameters via dictionary
        if params:
            for key in params:
                clientObject[key] = params[key]

        # Delete None attributes for improved performance
        deleteEmptyAttributes(clientObject)

        # Add Member Rotational Restraint to client model
        model.clientModel.service.set_member_rotational_restraint(clientObject)



from RSTAB.initModel import Model, clearAttributes, deleteEmptyAttributes, ConvertToDlString
from RSTAB.enums import ImperfectionType, ImperfectionCaseDirection, DirectionForLevelDirection
from RSTAB.enums import ImperfectionCaseSourceType, ImperfectionCaseAssignmentType

class ImperfectionCase():

    level_imperfection_item: dict = {'no':1,'level':3,'e_1':0,'theta_1':0,'e_2':0,'theta_2':0,'comment':''}
    imperfection_case_item: dict = {'no':1,'name':1, 'factor':1.1,'operator_type': 'OPERATOR_NONE','comment':''}

    def __init__(self,
                 no: int = 1,
                 type = ImperfectionType.IMPERFECTION_TYPE_LOCAL_IMPERFECTIONS,
                 assigned_to_load_cases: str = '',
                 assigned_to_load_combinations: str = '',
                 assign_to_combinations_without_assigned_imperfection_case: bool = True,
                 active: bool = True,
                 comment: str = '',
                 params: dict = None,
                 model = Model):
        """
        Imperfection Case

        Args:
            no (int): Imperfection Case Tag
            type (enum): Imperfection Type
            assigned_to_load_cases (str, optional): Assigned to Load Case
            assigned_to_load_combinations (str, optional): Assigned to Load Combinations
            assign_to_combinations_without_assigned_imperfection_case (bool): Assign to all Load Combinations without Assigned Imperfection Case
            active (bool): Active
            comment (str, optional): Comments
            params (dict, optional): Any WS Parameter relevant to the object and its value in form of a dictionary
            model (RSTAB Class, optional): Model to be edited
        """

        # Client model | Imperfection Case
        clientObject = model.clientModel.factory.create('ns0:imperfection_case')

        # Clears object atributes | Sets all atributes to None
        clearAttributes(clientObject)

        # Imperfection Case No.
        clientObject.no = no

        # Type of imperfection
        clientObject.type = type.name

        # Assign to Load Combinations
        if assigned_to_load_combinations:
            clientObject.assigned_to_load_combinations = ConvertToDlString(assigned_to_load_combinations)

        # Assign to Load Cases
        if assigned_to_load_cases:
            clientObject.assigned_to_load_cases = ConvertToDlString(assigned_to_load_cases)

        # Assign to Combinations Without Assigned Imperfection Case
        clientObject.assign_to_combinations_without_assigned_imperfection_case = assign_to_combinations_without_assigned_imperfection_case

        # Active
        clientObject.is_active = active

        # Comment
        clientObject.comment = comment

        # Adding optional parameters via dictionary
        if params:
            for key in params:
                clientObject[key] = params[key]

        # Delete None attributes for improved performance
        deleteEmptyAttributes(clientObject)

        # Add Imperfection Case to client model
        model.clientModel.service.set_imperfection_case(clientObject)

    @staticmethod
    def Local(
              no: int = 1,
              assigned_to_load_combinations: str = '',
              assigned_to_load_cases: str = '',
              comment: str = '',
              params: dict = None,
              model = Model):
        """
        Imperfection Case Local Imperfections

        Args:
            no (int): Imperfection Case Tag
            assigned_to_load_combinations (str, optional): Assigned Load Combinations
            assigned_to_load_cases (str, optional): Assigned Load Cases
            comment (str, optional): Comments
            params (dict, optional): Any WS Parameter relevant to the object and its value in form of a dictionary
            model (RSTAB Class, optional): Model to be edited
        """

        # assigned_to_load_combinations XY is first in parameter list because imperfections are
        # usually assigned to load combinations.

        # Client model | Imperfection Case
        clientObject = model.clientModel.factory.create('ns0:imperfection_case')

        # Clears object atributes | Sets all atributes to None
        clearAttributes(clientObject)

        # Imperfection Case No.
        clientObject.no = no

        # Local Type
        clientObject.type = ImperfectionType.IMPERFECTION_TYPE_LOCAL_IMPERFECTIONS.name

        # Assign to Load Combinations
        if assigned_to_load_combinations:
            clientObject.assigned_to_load_combinations = ConvertToDlString(assigned_to_load_combinations)

        # Assign to Load Cases
        if assigned_to_load_cases:
            clientObject.assigned_to_load_cases = ConvertToDlString(assigned_to_load_cases)

        if len(assigned_to_load_cases) + len(assigned_to_load_combinations) == 0:
            print('Warning: An imperfection case should be assigned to a load case or load combination.')

        # Comment
        clientObject.comment = comment

        # Adding optional parameters via dictionary
        if params:
            for key in params:
                clientObject[key] = params[key]

        # Delete None attributes for improved performance
        deleteEmptyAttributes(clientObject)

        # Add Imperfection Case to client model
        model.clientModel.service.set_imperfection_case(clientObject)

    @staticmethod
    def InitialSwayViaTable(
                            no: int = 1,
                            assigned_to_load_cases: str = '',
                            assigned_to_load_combinations: str = '',
                            assign_to_combinations_without_assigned_imperfection_case: bool = True,
                            direction = ImperfectionCaseDirection.IMPERFECTION_CASE_DIRECTION_GLOBAL_Z_OR_USER_DEFINED_W_TRUE,
                            direction_for_level_direction = DirectionForLevelDirection.DIRECTION_X,
                            coordinate_system: int = 1,
                            sway_coefficients_reciprocal: bool = True,
                            level_imperfections: list = [level_imperfection_item],
                            active: bool = True,
                            comment: str = '',
                            params: dict = None,
                            model = Model):
        """
        Imperfection Case Initial Sway via Table

        Args:
            no (int): Imperfection Case Tag
            assigned_to_load_cases (str, optional): Assigned to Load Case
            assigned_to_load_combinations (str, optional): Assigned to Load Combinations
            assign_to_combinations_without_assigned_imperfection_case (bool): Assign to all Load Combinations without assigned Imperfection Case
            direction (enum): Imperfection Case Direction Enumeration
            direction_for_level_direction (enum): Direction For Level Direction Enumeration
            coordinate_system (int): Coordinate system
            sway_coefficients_reciprocal (bool): Sway Coefficient as reciprocal of 1
            level_imperfections (list): Level Imperfections
            active (bool): Active
            comment (str, optional): Comments
            params (dict, optional): Any WS Parameter relevant to the object and its value in form of a dictionary
            model (RSTAB Class, optional): Model to be edited
        """

        # Client model | Imperfection Case
        clientObject = model.clientModel.factory.create('ns0:imperfection_case')

        # Clears object atributes | Sets all atributes to None
        clearAttributes(clientObject)

        # Imperfection Case No.
        clientObject.no = no

        # Type of imperfection
        clientObject.type = ImperfectionType.IMPERFECTION_TYPE_INITIAL_SWAY_VIA_TABLE.name

        # Assign to Load Combinations
        if assigned_to_load_combinations:
            clientObject.assigned_to_load_combinations = ConvertToDlString(assigned_to_load_combinations)

        # Assign to Load Cases
        if assigned_to_load_cases:
            clientObject.assigned_to_load_cases = ConvertToDlString(assigned_to_load_cases)

        # Assign to Combinations Without Assigned Imperfection Case
        clientObject.assign_to_combinations_without_assigned_imperfection_case = assign_to_combinations_without_assigned_imperfection_case

        # Level Direction
        clientObject.direction = direction.name

        # Imperfection Direction
        clientObject.direction_for_level_direction = direction_for_level_direction.name

        # Coordinate System
        clientObject.coordinate_system = coordinate_system

        # Sway Coeefficient as Reciprocal of 1
        clientObject.sway_coefficients_reciprocal = sway_coefficients_reciprocal

        # Level Imperfections Table
        clientObject.level_imperfections = model.clientModel.factory.create('ns0:array_of_imperfection_case_level_imperfections')

        for i in level_imperfections:
            li_proto = model.clientModel.factory.create('ns0:imperfection_case_level_imperfections_row')
            li_proto.no = i['no']
            li_proto.row = model.clientModel.factory.create('ns0:imperfection_case_level_imperfections')
            clearAttributes(li_proto.row)
            li_proto.row.level = i['level']
            li_proto.row.e_1 = i['e_1']
            li_proto.row.theta_1 = i['theta_1']
            li_proto.row.e_2 = i['e_2']
            li_proto.row.theta_2 = i['theta_2']
            li_proto.row.comment = i['comment']

            clientObject.level_imperfections.imperfection_case_level_imperfections.append(li_proto)

        # Active
        clientObject.is_active = active

        # Comment
        clientObject.comment = comment

        # Adding optional parameters via dictionary
        if params:
            for key in params:
                clientObject[key] = params[key]

        # Delete None attributes for improved performance
        deleteEmptyAttributes(clientObject)

        # Add Imperfection Case to client model
        model.clientModel.service.set_imperfection_case(clientObject)

    @staticmethod
    def NotionalLoads(
                 no: int = 1,
                 assigned_to_load_cases: str = '',
                 assigned_to_load_combinations: str = '',
                 assign_to_combinations_without_assigned_imperfection_case: bool = True,
                 load_case_for_notional_loads: int = 1,
                 active: bool = True,
                 comment: str = '',
                 params: dict = None,
                 model = Model):
        """
        Imperfection Case

        Args:
            no (int): Imperfection Case Tag
            assigned_to_load_cases (str, optional): Assigned to Load Case
            assigned_to_load_combinations (str, optional): Assigned to Load Combinations
            assign_to_combinations_without_assigned_imperfection_case (bool): Assign to all Load Combinations without assigned Imperfection Case
            load_case_for_notional_loads(int): Load Case for Notional Loads
            active (bool): Active
            comment (str, optional): Comments
            params (dict, optional): Any WS Parameter relevant to the object and its value in form of a dictionary
            model (RSTAB Class, optional): Model to be edited
        """

        # Client model | Imperfection Case
        clientObject = model.clientModel.factory.create('ns0:imperfection_case')

        # Clears object atributes | Sets all atributes to None
        clearAttributes(clientObject)

        # Imperfection Case No.
        clientObject.no = no

        # Type of imperfection
        clientObject.type = ImperfectionType.IMPERFECTION_TYPE_NOTIONAL_LOADS_FROM_LOAD_CASE.name

        # Assign to Load Combinations
        if assigned_to_load_combinations:
            clientObject.assigned_to_load_combinations = ConvertToDlString(assigned_to_load_combinations)

        # Assign to Load Cases
        if assigned_to_load_cases:
            clientObject.assigned_to_load_cases = ConvertToDlString(assigned_to_load_cases)

        # Assign to Combinations Without Assigned Imperfection Case
        clientObject.assign_to_combinations_without_assigned_imperfection_case = assign_to_combinations_without_assigned_imperfection_case

        # Load Case for Notional Loads
        clientObject.load_case_for_notional_loads = load_case_for_notional_loads

        # Active
        clientObject.is_active = active

        # Comment
        clientObject.comment = comment

        # Adding optional parameters via dictionary
        if params:
            for key in params:
                clientObject[key] = params[key]

        # Delete None attributes for improved performance
        deleteEmptyAttributes(clientObject)

        # Add Imperfection Case to client model
        model.clientModel.service.set_imperfection_case(clientObject)

    @staticmethod
    def StaticDeformation(
                 no: int = 1,
                 assigned_to_load_cases: str = '',
                 assigned_to_load_combinations: str = '',
                 assign_to_combinations_without_assigned_imperfection_case: bool = True,
                 direction = ImperfectionCaseDirection.IMPERFECTION_CASE_DIRECTION_GLOBAL_Z_OR_USER_DEFINED_W_TRUE,
                 coordinate_system: int = 1,
                 source = ImperfectionCaseSourceType.SOURCE_TYPE_LOAD_CASE,
                 imperfection_shape_from: int = 1,
                 imperfection_magnitude: float = 0.3,
                 magnitude_assignment_type = ImperfectionCaseAssignmentType.MAGNITUDE_ASSIGNMENT_SPECIFIC_NODE,
                 reference_node: int = None,
                 active: bool = True,
                 comment: str = '',
                 params: dict = None,
                 model = Model):
        """
        Static Deformation Type

        Args:
            no (int): Imperfection Case Tag
            assigned_to_load_cases (str, optional): Assigned to Load Case
            assigned_to_load_combinations (str, optional): Assigned to Load Combinations
            assign_to_combinations_without_assigned_imperfection_case (bool): Assign to all Load Combinations without assigned Imperfection Case
            direction (enum): Imperfection Case Direction Enumeration
            coordinate_system (int): Coordinate System
            source (enum): Imperfection Case Source Type Enumeration
            imperfection_shape_from (int): Imperfection Shape From
            imperfection_magnitude (float): Imperfection Magnitude
            magnitude_assignment_type (enum): Imperfection Case Assignment Type Enumeration
            reference_node (int): Reference Node
            active (bool): Active
            comment (str, optional): Comments
            params (dict, optional): Any WS Parameter relevant to the object and its value in form of a dictionary
            model (RSTAB Class, optional): Model to be edited
        """

        # Client model | Imperfection Case
        clientObject = model.clientModel.factory.create('ns0:imperfection_case')

        # Clears object atributes | Sets all atributes to None
        clearAttributes(clientObject)

        # Imperfection Case No.
        clientObject.no = no

        # Type of imperfection
        clientObject.type = ImperfectionType.IMPERFECTION_TYPE_STATIC_DEFORMATION.name

        # Assign to Load Combinations
        if assigned_to_load_combinations:
            clientObject.assigned_to_load_combinations = ConvertToDlString(assigned_to_load_combinations)

        # Assign to Load Cases
        if assigned_to_load_cases:
            clientObject.assigned_to_load_cases = ConvertToDlString(assigned_to_load_cases)

        # Assign to Combinations Without Assigned Imperfection Case
        clientObject.assign_to_combinations_without_assigned_imperfection_case = assign_to_combinations_without_assigned_imperfection_case

        # Direction
        clientObject.direction = direction.name

        # Coordinate System number
        clientObject.coordinate_system = coordinate_system

        # Source
        clientObject.source = source.name

        # Shape from Load Case/Combination
        if source == ImperfectionCaseSourceType.SOURCE_TYPE_LOAD_CASE:
            clientObject.shape_from_load_case = imperfection_shape_from
        else:
            clientObject.shape_from_load_combination = imperfection_shape_from

        # Imperfection Magnuítude
        clientObject.delta_zero = imperfection_magnitude

        # Reference Location
        clientObject.magnitude_assignment_type = magnitude_assignment_type.name

        # Reference Node
        if magnitude_assignment_type == ImperfectionCaseAssignmentType.MAGNITUDE_ASSIGNMENT_SPECIFIC_NODE:
            clientObject.reference_node = reference_node

        # Active
        clientObject.is_active = active

        # Comment
        clientObject.comment = comment

        # Adding optional parameters via dictionary
        if params:
            for key in params:
                clientObject[key] = params[key]

        # Delete None attributes for improved performance
        deleteEmptyAttributes(clientObject)

        # Add Imperfection Case to client model
        model.clientModel.service.set_imperfection_case(clientObject)

    @staticmethod
    def Group(
                 no: int = 1,
                 assigned_to_load_cases: str = '',
                 assigned_to_load_combinations: str = '',
                 assign_to_combinations_without_assigned_imperfection_case: bool = True,
                 imperfection_cases: list = [imperfection_case_item],
                 active: bool = True,
                 comment: str = '',
                 params: dict = None,
                 model = Model):
        """
        Imperfection Case

        Args:
            no (int): Imperfection Case Tag
            assigned_to_load_cases (str, optional): Assigned to Load Case
            assigned_to_load_combinations (str, optional): Assigned to Load Combinations
            assign_to_combinations_without_assigned_imperfection_case (bool): Assign to all Load Combinations without assigned Imperfection Case
            imperfection_cases (list): Imperfection Cases items
            active (bool): Active
            comment (str, optional): Comments
            params (dict, optional): Any WS Parameter relevant to the object and its value in form of a dictionary
            model (RSTAB Class, optional): Model to be edited
        """

        # Client model | Imperfection Case
        clientObject = model.clientModel.factory.create('ns0:imperfection_case')

        # Clears object atributes | Sets all atributes to None
        clearAttributes(clientObject)

        # Imperfection Case No.
        clientObject.no = no

        # Type
        clientObject.type = ImperfectionType.IMPERFECTION_TYPE_IMPERFECTION_CASES_GROUP.name

        # Assign to Load Combinations
        if assigned_to_load_combinations:
            clientObject.assigned_to_load_combinations = ConvertToDlString(assigned_to_load_combinations)

        # Assign to Load Cases
        if assigned_to_load_cases:
            clientObject.assigned_to_load_cases = ConvertToDlString(assigned_to_load_cases)

        # Assign to Combinations Without Assigned Imperfection Case
        clientObject.assign_to_combinations_without_assigned_imperfection_case = assign_to_combinations_without_assigned_imperfection_case

        # Imperfection Case Items
        clientObject.imperfection_cases_items = model.clientModel.factory.create('ns0:array_of_imperfection_case_imperfection_cases_items')

        for i in imperfection_cases:
            li_proto = model.clientModel.factory.create('ns0:imperfection_case_imperfection_cases_items_row')
            li_proto.no = i['no']
            li_proto.row = model.clientModel.factory.create('ns0:imperfection_case_imperfection_cases_items')
            clearAttributes(li_proto.row)
            li_proto.row.name = i['name']
            li_proto.row.factor = i['factor']
            li_proto.row.operator_type = i['operator_type']
            li_proto.row.comment = i['comment']

            clientObject.imperfection_cases_items.imperfection_case_imperfection_cases_items.append(li_proto)

        # Active
        clientObject.is_active = active

        # Comment
        clientObject.comment = comment

        # Adding optional parameters via dictionary
        if params:
            for key in params:
                clientObject[key] = params[key]

        # Delete None attributes for improved performance
        deleteEmptyAttributes(clientObject)

        # Add Imperfection Case to client model
        model.clientModel.service.set_imperfection_case(clientObject)



from setuptools import find_packages
from setuptools import setup
import pathlib

here = pathlib.Path(__file__).parent
readme = (here/"README.md").read_text(encoding="utf-8")

setup(
    name='RSTAB',
    version='1.10.3',
    description='Python Framework for RSTAB9 Web Services',
    long_description=readme,
    long_description_content_type = "text/markdown",
    url="https://github.com/Dlubal-Software/RSTAB_Python_Client",
    author="Dlubal Software",
    author_email="info@dlubal.com",
    license="MIT",
    classifiers=[
        "License :: OSI Approved :: MIT License",
        "Programming Language :: Python :: 3",
        "Programming Language :: Python :: 3.10",
        "Programming Language :: Python :: 3.11"
    ],
    packages=find_packages(),
    package_dir={"RSTAB":"RSTAB"},
    include_package_data=True,
    install_requires=["requests", "six", "suds-py3", "xmltodict", "pytest", "psutil", "mock", "setuptools"],
    zip_safe = False
)



import xmltodict
import csv
from RSTAB.initModel import Model
from RSTAB.enums import IFCExportType, ObjectTypes

def ExportDetailsOfDesignToCSV(targetDirectoryPath: str, model = Model):
    '''
    Export details of design to CSV format.

    Args:
        targetDirectoryPath (string): Destination path to the directory
    '''
    model.clientModel.service.export_details_of_design_to_csv(targetDirectoryPath)

def ExportResultTablesToCSV(targetDirectoryPath: str, model = Model):
    '''
    Export result tables to CSV format.

    Args:
        targetDirectoryPath (string): Destination path to the directory
    '''
    model.clientModel.service.export_result_tables_to_csv(targetDirectoryPath)

def ExportResultTablesToXML(targetFilePath: str, model = Model):
    '''
    Export result tables to XML format.

    Args:
        targetFilePath (string): Destination path to the file
    '''
    model.clientModel.service.export_result_tables_to_xml(targetFilePath)

def ExportResultTablesWithDetailedMembersResultsToCSV(targetDirectoryPath: str, model = Model):
    '''
    Export result tables with detailed member results to CSV format.

    Args:
        targetDirectoryPath (string): Destination path to the directory
    '''
    model.clientModel.service.export_result_tables_with_detailed_members_results_to_csv(targetDirectoryPath)

def ExportResultTablesWithDetailedMembersResultsToXML(targetFilePath: str, model = Model):
    '''
    Export result tables with detailed member results to XML format.

    Args:
        targetFilePath (string): Destination path to the file
    '''
    model.clientModel.service.export_result_tables_with_detailed_members_results_to_xml(targetFilePath)

def ExportTo(targetFilePath: str, model = Model):
    '''
    Export active model to format specified by suffix of the destination path to file.
    Supported formats are .xml, .vtk, .xlsx, .saf, .gltf, and .glb.

    Args:
        targetFilePath (string): Destination path to the file
    '''
    model.clientModel.service.export_to(targetFilePath)

IFCExportSettings = {
    'mirror_axis_x': False,
    'mirror_axis_y': False,
    'mirror_axis_z': False,
    'origin_coordinate_x': 0, # float
    'origin_coordinate_y': 0, # float
    'origin_coordinate_z': 0, # float
    'axis_rotation_sequence': 'X_Y_Z', # string
    'rotation_angle_0': 0, # float
    'rotation_angle_1': 0, # float
    'rotation_angle_2': 0, # float
    'switch_axis_x': 'X', # 'X','Y' or 'Z'
    'switch_axis_y': 'Y', # 'X','Y' or 'Z'
    'switch_axis_z': 'Z', # 'X','Y' or 'Z'
    'remove_accents': False,
    'export_type': IFCExportType.E_EXPORT_IFC4_REFERENCE_VIEW.name # export to ifc type
}

ObjectLocation = {
    'type':ObjectTypes.E_OBJECT_TYPE_MEMBER.name,
    'no': 1,
    'parent_no': 1
}
'''
Args:
    type (enum): Type of object to be exported to IFC
    no (int): Number of the object to be exoprted to IFC
    parent_no (int): Parrent number  of the object to be exoprted to IFC, if there is one. Usually 1.
'''

def ObjectLocations(locationsArray):
    '''
    Use this function to obtain 3rd parameter of ExportToIFC() function.
    If left out, all objects will be exported to IFC.

    Args:
        locationsVector (array): Array of ObjectLocation variables

    {'location': {'type':ObjectTypes.E_OBJECT_TYPE_MEMBER.name, 'no': 1, 'parent_no': 1}}
    '''

    return {'location': locationsArray}


def ExportToIFC(targetFilePath: str, IFCSettings = IFCExportSettings, ObjectLoc = None, model = Model):
    '''
    Use this function to export active model t o IFC.

    Args:
        targetFilePath (string): Target file path incl. suffix.
        IFCSettings (IFCExportSettings): IFC export settings dictionary.
        ObjectLoc (array): array of ObjectLocation dictionary. If left out, all objects will be exported to IFC.
    '''
    if ObjectLoc:
        model.clientModel.service.export_to_ifc(targetFilePath, IFCSettings, ObjectLoc)
    else:
        model.clientModel.service.export_to_ifc(targetFilePath, IFCSettings)

def ExportToTables(targetDirectoryPath: str, model = Model):
    '''
    Export active model to tables.

    Args:
        targetDirectoryPath (string): Destination path to the directory
    '''
    model.clientModel.service.export_to_tables(targetDirectoryPath)

def GetTableExportConfigManager(model = Model):
    '''
    Export active model to tables.
    Use this fuction to obtain input parameter to SetTableExportConfigManager()

    Return:
        Table export configuration.
    '''
    return model.clientModel.service.get_table_export_config_manager()

def SetTableExportConfigManager(TableExportConfigManager, model = Model):
    '''
    Create or change table export settings.
    To obtain input data (TableExportConfigManager) call GetTableExportConfigManager() first.

    Args:
        TableExportConfigManager (dict): Table export config
    '''
    model.clientModel.service.set_table_export_config_manager(TableExportConfigManager)

def ParseCSVResultsFromSelectedFileToDict(filePath: str):

    # Using encoding parameter ensures proper data translation, leaving out BOM etc.
    # TODO: fix the value assigment; it only works with simple one-line header
    #       consider all corner cases
    with open(filePath, mode='r', encoding='utf-8-sig') as f:
        reader = csv.DictReader(f,delimiter=';')
        my_dictionary = []
        for line in reader:
            my_dictionary.append(line)
    return my_dictionary

def ParseXMLResultsFromSelectedFileToDict(filePath: str):

    with open(filePath, "rb") as f:
        my_dictionary = xmltodict.parse(f, xml_attribs=True)
    return my_dictionary



from RSTAB.initModel import Model, clearAttributes, deleteEmptyAttributes

class StructureModification():
    material_item = {'no': 1, 'material_name': 1, 'modification_type': 'DIVISION_FACTOR', 'E_and_G': 1.5, 'comment': 'comment'}
    section_item = {'no': 1, 'section_name': 'IPN 300', 'A': 1.0, 'A_y':1.0, 'A_z': 1.0, 'J': 1.0, 'I_y': 1.0, 'I_z': 1.0}
    member_item = {'no': 2, 'member_modification': 1, 'members':'11', 'comment': ''}
    member_hinge_item = {'no': 1, 'member_side': 'Start', 'C_u_x': 1, 'C_u_y': 0, 'C_u_z': 0, 'C_phi_x': 0, 'C_phi_y': 0, 'C_phi_z': 0}
    nodal_support_item = {'C_u_X': 1, 'C_u_Y': 0, 'C_u_Z': 0, 'C_phi_X': 0, 'C_phi_Y': 0, 'C_phi_Z': 0}
    member_support_item = {'C_u_x': 1, 'C_u_y': 0, 'C_u_z': 0, 'C_s_x': 0, 'C_s_y': 0, 'C_s_z': 0, 'C_phi_x': 0}
    modify_stiffness = {'modify_stiffnesses_gamma_m': False,
                        'modify_stiffnesses_materials': False,
                        'modify_stiffnesses_sections': False,
                        'modify_stiffnesses_members': False,
                        'modify_stiffnesses_member_hinges': False,
                        'modify_stiffnesses_nodal_supports': False,
                        'modify_stiffnesses_member_supports': False,
                        'modify_stiffness_timber_members_due_moisture_class': False,
                        'nonlinearities_disabled_material_nonlinearity_models': False,
                        'nonlinearities_disabled_material_temperature_nonlinearities': False,
                        'nonlinearities_disabled_member_types': False,
                        'nonlinearities_disabled_member_nonlinearities': False,
                        'nonlinearities_disabled_member_hinges': False,
                        'nonlinearities_disabled_nodal_supports': False,
                        'nonlinearities_disabled_member_supports': False,
                        'deactivate_members_enabled': False}

    def __init__(self,
                 no: int = 1,
                 modify_stiffnesses: dict = modify_stiffness,
                 modify_stiffnesses_materials_list: list = None,
                 modify_stiffnesses_sections_list: list = None,
                 modify_stiffnesses_members_list: list = None,
                 modify_stiffnesses_member_hinges_list: list = None,
                 modify_stiffnesses_nodal_supports_list: list = None,
                 modify_stiffnesses_member_supports_list: list = None,
                 comment: str = '',
                 params: dict = None,
                 model = Model):
        """
        Structural Modification
        Modify only objects allready existing and used/assigned in the model.

        Args:
            no (int, optional): Structure Modification Tag
            modify_stiffnesses (dict, optional): Modify Stiffnesses
            modify_stiffnesses_materials_list (list, optional): Modify Stiffnesses Materials List
            modify_stiffnesses_sections_list (list, optional): Modify Stiffnesses Sections List
            modify_stiffnesses_members_list (list, optional): Modify Stiffnesses Members List
            modify_stiffnesses_member_hinges_list (list, optional): Modify Stiffnesses Member Hinges List
            modify_stiffnesses_nodal_supports_list (list, optional): Modify Stiffnesses Nodal Supports List
            modify_stiffnesses_member_supports_list (list, optional): Modify Stiffnesses Member Suppoorts List
            comment (str, optional): Comment
            params (dict, optional): Any WS Parameter relevant to the object and its value in form of a dictionary
            model (RSTAB Class, optional): Model to be edited
        """

        # Client model | Structure Modification
        clientObject = None
        try:
            clientObject = model.clientModel.service.get_structure_modification(no)
        except:
            clientObject = model.clientModel.factory.create('ns0:structure_modification')

            # Clears object atributes | Sets all atributes to None
            clearAttributes(clientObject)

            # Structure Modification No.
            clientObject.no = no

            # Modify Stiffneesses
            clientObject.modify_stiffnesses_gamma_m = modify_stiffnesses['modify_stiffnesses_gamma_m']
            clientObject.modify_stiffnesses_materials = modify_stiffnesses['modify_stiffnesses_materials']
            clientObject.modify_stiffnesses_sections = modify_stiffnesses['modify_stiffnesses_sections']
            clientObject.modify_stiffnesses_members = modify_stiffnesses['modify_stiffnesses_members']
            clientObject.modify_stiffnesses_member_hinges = modify_stiffnesses['modify_stiffnesses_member_hinges']
            clientObject.modify_stiffnesses_nodal_supports = modify_stiffnesses['modify_stiffnesses_nodal_supports']
            clientObject.modify_stiffnesses_member_supports = modify_stiffnesses['modify_stiffnesses_member_supports']
            clientObject.modify_stiffness_timber_members_due_moisture_class = modify_stiffnesses['modify_stiffness_timber_members_due_moisture_class']
            clientObject.nonlinearities_disabled_material_nonlinearity_models = modify_stiffnesses['nonlinearities_disabled_material_nonlinearity_models']
            clientObject.nonlinearities_disabled_material_temperature_nonlinearities = modify_stiffnesses['nonlinearities_disabled_material_temperature_nonlinearities']
            clientObject.nonlinearities_disabled_member_types = modify_stiffnesses['nonlinearities_disabled_member_types']
            clientObject.nonlinearities_disabled_member_nonlinearities = modify_stiffnesses['nonlinearities_disabled_member_nonlinearities']
            clientObject.nonlinearities_disabled_member_hinges = modify_stiffnesses['nonlinearities_disabled_member_hinges']
            clientObject.nonlinearities_disabled_nodal_supports = modify_stiffnesses['nonlinearities_disabled_nodal_supports']
            clientObject.nonlinearities_disabled_member_supports = modify_stiffnesses['nonlinearities_disabled_member_supports']
            clientObject.deactivate_members_enabled = modify_stiffnesses['deactivate_members_enabled']

            model.clientModel.service.set_structure_modification(clientObject)
            clientObject = model.clientModel.service.get_structure_modification(no)

        # Modify Stiffneesses Tables
        if modify_stiffnesses['modify_stiffnesses_materials']:
            for i in modify_stiffnesses_materials_list:
                idx = i['no']-1
                clientObject.modify_stiffnesses_material_table[0][idx].row.modification_type = i['modification_type']
                clientObject.modify_stiffnesses_material_table[0][idx].row.E_and_G = i['E_and_G']
                clientObject.modify_stiffnesses_material_table[0][idx].row.comment = i['comment']
        if modify_stiffnesses['modify_stiffnesses_sections']:
            for i in modify_stiffnesses_sections_list:
                idx = i['no']-1
                clientObject.modify_stiffnesses_section_table[0][idx].row.A = i['A']
                clientObject.modify_stiffnesses_section_table[0][idx].row.A_y = i['A_y']
                clientObject.modify_stiffnesses_section_table[0][idx].row.A_z = i['A_z']
                clientObject.modify_stiffnesses_section_table[0][idx].row.J = i['J']
                clientObject.modify_stiffnesses_section_table[0][idx].row.I_y = i['I_y']
                clientObject.modify_stiffnesses_section_table[0][idx].row.I_z = i['I_z']
        if modify_stiffnesses['modify_stiffnesses_members']:
            for i in modify_stiffnesses_members_list:
                idx = i['no']-1
                clientObject.modify_stiffnesses_members_table[0][idx].row.member_modification = i['member_modification']
                clientObject.modify_stiffnesses_members_table[0][idx].row.members = i['members']
                clientObject.modify_stiffnesses_members_table[0][idx].row.comment = i['comment']
        if modify_stiffnesses['modify_stiffnesses_member_hinges']:
            for i in modify_stiffnesses_member_hinges_list:
                idx = i['no']-1
                clientObject.modify_stiffnesses_member_hinges_table[0][idx].row.member_side = i['member_side']
                clientObject.modify_stiffnesses_member_hinges_table[0][idx].row.C_u_x = i['C_u_x']
                clientObject.modify_stiffnesses_member_hinges_table[0][idx].row.C_u_y = i['C_u_y']
                clientObject.modify_stiffnesses_member_hinges_table[0][idx].row.C_u_z = i['C_u_z']
                clientObject.modify_stiffnesses_member_hinges_table[0][idx].row.C_phi_x = i['C_phi_x']
                clientObject.modify_stiffnesses_member_hinges_table[0][idx].row.C_phi_y = i['C_phi_y']
                clientObject.modify_stiffnesses_member_hinges_table[0][idx].row.C_phi_z = i['C_phi_z']
        if modify_stiffnesses['modify_stiffnesses_nodal_supports']:
            for i in modify_stiffnesses_nodal_supports_list:
                idx = i['no']-1
                clientObject.modify_stiffnesses_nodal_supports_table[0][idx].row.C_u_x = i['C_u_x']
                clientObject.modify_stiffnesses_nodal_supports_table[0][idx].row.C_u_y = i['C_u_y']
                clientObject.modify_stiffnesses_nodal_supports_table[0][idx].row.C_u_z = i['C_u_z']
                clientObject.modify_stiffnesses_nodal_supports_table[0][idx].row.C_phi_x = i['C_phi_x']
                clientObject.modify_stiffnesses_nodal_supports_table[0][idx].row.C_phi_y = i['C_phi_y']
                clientObject.modify_stiffnesses_nodal_supports_table[0][idx].row.C_phi_z = i['C_phi_z']
        if modify_stiffnesses['modify_stiffnesses_member_supports']:
            for i in modify_stiffnesses_member_supports_list:
                idx = i['no']-1
                clientObject.modify_stiffnesses_member_supports_table[0][idx].row.C_u_x = i['C_u_x']
                clientObject.modify_stiffnesses_member_supports_table[0][idx].row.C_u_y = i['C_u_y']
                clientObject.modify_stiffnesses_member_supports_table[0][idx].row.C_u_z = i['C_u_z']
                clientObject.modify_stiffnesses_member_supports_table[0][idx].row.C_s_x = i['C_s_x']
                clientObject.modify_stiffnesses_member_supports_table[0][idx].row.C_s_y = i['C_s_y']
                clientObject.modify_stiffnesses_member_supports_table[0][idx].row.C_s_z = i['C_s_z']
                clientObject.modify_stiffnesses_member_supports_table[0][idx].row.C_phi_x = i['C_phi_x']

        # Comment
        clientObject.comment = comment

        # Adding optional parameters via dictionary
        if params:
            for key in params:
                clientObject[key] = params[key]

        # Delete None attributes for improved performance
        deleteEmptyAttributes(clientObject)

        # Add Structure Modification to client model
        model.clientModel.service.set_structure_modification(clientObject)




url = "http://127.0.0.1"
port = "8081"
connected = False
api_key = None
verify = True # This can be a boolean or a path to a CA bundle file (in case of self-signed certificate it's required)

# Is filled in the initModel
client = None
ca = None
cacheLoc = ""
session = None



from RSTAB.initModel import Model

class PrintoutReport():
    """
    Printout report class encopassing available printout report methods.

    TODO: Create printout report US is paused US-8034.
    """

    @staticmethod
    def delete(id_list, model = Model):
        """
        Delete printout report
        """
        model.clientModel.service.delete_printout_reports(id_list)

    @staticmethod
    def exportToHTML(report_id: int, target_file_path: str, model = Model):
        """
        Export printout report to a HTML.
        """
        model.clientModel.service.export_printout_report_to_html(report_id, target_file_path)

    @staticmethod
    def exportToPDF(report_id: int, target_file_path: str, model = Model):
        """
        Export printout report to a PDF.
        """
        model.clientModel.service.export_printout_report_to_pdf(report_id, target_file_path)

    @staticmethod
    def getList(model = Model):
        """
        Get list of printout reports.
        """
        return model.clientModel.service.get_list_of_printout_reports()[0]

    @staticmethod
    def print(printout_report_id: int = 1, model = Model):
        """
        Print printout report.
        """
        model.clientModel.service.print_printout_report(printout_report_id)



from RSTAB.initModel import Model, clearAttributes
from RSTAB.enums import ModelType

class ModelParameters():

    def __init__(self,
                 model_parameters: list = None,
                 model = Model):

        '''
        Args:
            model_parameters (list of lists): Model Parameters Table
                model_parameters = [[name, description_1, description_2],...]
            model (RSTAB Class, optional): Model to be edited
        '''

        # Client Model | Get Model Parameters
        clientObject = model.clientModel.service.get_model_parameters()

        # Add Model Parameters

        for i,j in enumerate(model_parameters):
            mp = model.clientModel.factory.create('ns0:model_parameters_row')
            mp.no = i+3
            mp.row = model.clientModel.factory.create('ns0:model_parameters')
            clearAttributes(mp.row)
            mp.row.name = model_parameters[i][0]
            mp.row.description_1 = model_parameters[i][1]
            mp.row.description_2 = model_parameters[i][2]

            clientObject.model_parameters.append(mp)

        # Add Base Data Model Parameters to client model
        model.clientModel.service.set_model_parameters(clientObject)

class ModelParametersLocation():

    def __init__(self,
                 model_parameters_location: list = None,
                 model = Model):

        '''
        Args:
            model_parameters_location (list of lists): Model Parameters Location Table
                model_parameters_location = [[ModelLocationRowType Enumeration, name, value, unit_group],...]
            model (RSTAB Class, optional): Model to be edited
        '''

        # Client Model | Get Model Parameters Location
        clientObject = model.clientModel.service.get_model_parameters_location()

        # Add Model Parameters

        for i,j in enumerate(model_parameters_location):
            mpl = model.clientModel.factory.create('ns0:model_parameters_location_row')
            mpl.no = i+1
            mpl.description = str(i+1)
            mpl.row = model.clientModel.factory.create('ns0:model_parameters_location')
            clearAttributes(mpl.row)
            mpl.row.location_row_type = model_parameters_location[i][0].name
            mpl.row.name = model_parameters_location[i][1]
            mpl.row.value = model_parameters_location[i][2]
            mpl.row.unit_group = model_parameters_location[i][3]

            clientObject.model_parameters_location.append(mpl)

        # Add Base Data Model Parameters to client model
        model.clientModel.service.set_model_parameters_location(clientObject)

class Modeltype():

    def __init__(self,
                 model_type = ModelType.E_MODEL_TYPE_3D,
                 model = Model):
        '''
        Args:
            model_type (enum): Model Type Enumeration
            model (RSTAB Class, optional): Model to be edited
        '''

        # Add Model Type to Client Model
        model.clientModel.service.set_model_type(model_type.name)



from RSTAB.initModel import Model, SetAddonStatus, clearAttributes, deleteEmptyAttributes
from RSTAB.enums import DirectionalComponentCombinationRule, PeriodicResponseCombinationRule, CqsDampingRule, AddOn

class SpectralAnalysisSettings():

    def __init__(self,
                 no: int = 1,
                 name: str = 'SRSS | SRSS',
                 periodic_combination = PeriodicResponseCombinationRule.SRSS,
                 directional_combination = DirectionalComponentCombinationRule.SRSS,
                 equivalent_linear_combination : bool = False,
                 save_mode_results : bool = False,
                 signed_dominant_mode_results : bool = False,
                 directional_component_scale_value : float = 0.3,
                 damping_for_cqc_rule = CqsDampingRule.CONSTANT_FOR_EACH_MODE,
                 constant_d_for_each_mode: float = 0.0,
                 comment: str = '',
                 params: dict = None,
                 model = Model):

        '''
        Args:
            no (int): Sprectral Analysis Settings Tag
            name (str): Sprectral Analysis Settings Name
            periodic_combination (enum): Periodic Combination Rule Enumeration
            directional_combination (enum): Directional Component Combination Rule Enumeration
            equivalent_linear_combination (bool): Equivalent Linear Combination Boolean
            save_mode_results (bool): Save Mode Results Boolean
            signed_dominant_mode_results (bool): Signed Dominant Mode Results Boolean
            directional_component_scale_value (float): Directional Component Scale Value
            damping_for_cqc_rule (enum): Cqs Damping Rule Enumeration
            constant_d_for_each_mode (float): Constant d for Each Mode
            comment (str, optional): Comments
            params (dict, optional): Any WS Parameter relevant to the object and its value in form of a dictionary
            model (RSTAB Class, optional): Model to be edited
        '''
        # Check if Spectral Add-on is active.
        SetAddonStatus(model.clientModel, AddOn.spectral_active)

        # Client model | Surface
        clientObject = model.clientModel.factory.create('ns0:spectral_analysis_settings')

        # Clears object atributes | Sets all atributes to None
        clearAttributes(clientObject)

        # Static Analysis Settings No.
        clientObject.no = no

        # Name
        if name:
            clientObject.name = name
            clientObject.user_defined_name_enabled = True

        # Periodic Combination
        clientObject.combination_rule_for_periodic_responses = periodic_combination.name

        # Directional Component
        clientObject.combination_rule_for_directional_components = directional_combination.name

        # Equivalent Linear Combination
        clientObject.use_equivalent_linear_combination = equivalent_linear_combination

        # Save Results of All Selected Modes
        clientObject.save_results_of_all_selected_modes = save_mode_results

        # TODO: Signed Results Using Dominant Mode are currently deactivated in RSTAB
        # Signed Results Using Dominant Mode
        clientObject.signed_results_using_dominant_mode = signed_dominant_mode_results
        '''
        if signed_dominant_mode_results:
            if directional_combination != DirectionalComponentCombinationRule.SCALED_SUM:
                raise ValueError("WARNING: Signed results using dominant mode is only available with Scaled Sum Directional Combination.")
        '''

        # Further Options
        if directional_combination == DirectionalComponentCombinationRule.SCALED_SUM:
            clientObject.combination_rule_for_directional_components_value = directional_component_scale_value

        if periodic_combination == PeriodicResponseCombinationRule.CQC:
            clientObject.damping_for_cqc_rule = damping_for_cqc_rule.name
            clientObject.constant_d_for_each_mode = constant_d_for_each_mode

        # Comment
        clientObject.comment = comment

        # Adding optional parameters via dictionary
        if params:
            for key in params:
                clientObject[key] = params[key]
        # Delete None attributes for improved performance
        deleteEmptyAttributes(clientObject)

        # Add Static Analysis Settings to client model
        model.clientModel.service.set_spectral_analysis_settings(clientObject)



from RSTAB.initModel import Model

def GetDesignOverview(model = Model):
    '''
    Returns whole Design Overview list.
    '''
    # Return Design Overview
    return model.clientModel.service.get_design_overview()

def GetPartialDesignOverview(comply: bool = False):
    """
    Returns part of Design Overview that do(esn't) comply.
    If comply == False, function resturns checks with Design Ration > 1
    If comply == True, function resturns checks with Design Ration  <= 1
    """
    designOverview = GetDesignOverview()

    designRatioLessThanOne = []
    designRationOverOne = []
    for do in designOverview[0]:
        if round(do.row['design_ratio'], 5) <= 1.0:
            designRatioLessThanOne.append(do[0])
        else:
            designRationOverOne.append(do[0])

    if comply:
        return designRatioLessThanOne
    else:
        return designRationOverOne





from RSTAB.initModel import Model, clearAttributes, deleteEmptyAttributes, ConvertToDlString
from RSTAB.enums import TimberMemberLocalSectionReductionType, MultipleOffsetDefinitionType, ZAxisReferenceType, OrientationType, DirectionType

class Components():
    def __init__(self,
                reduction_type: TimberMemberLocalSectionReductionType.REDUCTION_COMPONENT_TYPE_RECTANGLE_OPENING):
        '''
        Args:
            reduction_type (Enum): Timber Member Local Section Reduction Type Enum
        '''
        self.reduction_type = reduction_type.name
        if self.reduction_type == 'REDUCTION_COMPONENT_TYPE_RECTANGLE_OPENING':
            self.position = 1
            self.width = 0.5
            self.height = 0.5
            self.z_axis_reference_type = ZAxisReferenceType.E_POSITION_REFERENCE_CENTER.name
            self.distance = 0.01
            self.multiple = True
            self.multiple_number = 2
            self.multiple_offset_definition_type = MultipleOffsetDefinitionType.OFFSET_DEFINITION_TYPE_ABSOLUTE.name
            self.multiple_offset = 1
        elif self.reduction_type == 'REDUCTION_COMPONENT_TYPE_CIRCLE_OPENING':
            self.position = 1
            self.z_axis_reference_type = ZAxisReferenceType.E_POSITION_REFERENCE_TOP.name
            self.distance = 0.01
            self.diameter = 0.05
            self.multiple = True
            self.multiple_offset_definition_type = MultipleOffsetDefinitionType.OFFSET_DEFINITION_TYPE_ABSOLUTE.name
            self.multiple_number = 2
            self.multiple_offset = 2
        elif self.reduction_type == 'REDUCTION_COMPONENT_TYPE_START_NOTCH':
            self.length = 0.2
            self.orientation_type = OrientationType.E_ORIENTATION_DEPTH.name
            self.depth = 0.001
            self.width = None
            self.direction_type = DirectionType.E_DIRECTION_DEPTH_POSITIVE.name
            self.stability = True
            self.multiple = False
            self.fire_design = True
            self.fire_exposure_top = True
            self.fire_exposure_left = True
            self.fire_exposure_right = True
            self.fire_exposure_bottom = True
            self.support = True
        elif self.reduction_type == 'REDUCTION_COMPONENT_TYPE_INNER_NOTCH':
            self.position = 0.2
            self.length = 0.3
            self.orientation_type = OrientationType.E_ORIENTATION_DEPTH.name
            self.depth = 0.004
            self.direction_type = DirectionType.E_DIRECTION_DEPTH_POSITIVE.name
            self.stability = True
            self.multiple = True
            self.multiple_number = 2
            self.multiple_offset_definition_type = MultipleOffsetDefinitionType.OFFSET_DEFINITION_TYPE_ABSOLUTE.name
            self.multiple_offset = 2
            self.fire_design = True
            self.fire_exposure_top = True
            self.fire_exposure_left = True
            self.fire_exposure_right = True
            self.fire_exposure_bottom = True
        elif self.reduction_type == 'REDUCTION_COMPONENT_TYPE_END_NOTCH':
            self.length = 0.3
            self.orientation_type = OrientationType.E_ORIENTATION_DEPTH.name
            self.depth = 0.02
            self.direction_type = DirectionType.E_DIRECTION_DEPTH_POSITIVE.name
            self.stability = True
            self.support = True
            self.multiple = False
            self.fire_design = True
            self.fire_exposure_top = True
            self.fire_exposure_left = True
            self.fire_exposure_right = True
            self.fire_exposure_bottom = True
        else:
            assert True, 'Unsupported reduction_type'

class TimberMemberLocalSectionReduction():
    def __init__(self,
                 no: int = 1,
                 members: str = '1',
                 member_sets: str = '',
                 components: list = None,
                 name: str = '',
                 comment: str = '',
                 params: dict = None,
                 model = Model):
        """
        Args:
            no (int): Member Local Section Reduction Tag
            members (str): Assigned Members
            member_sets (str): Assigned Member Sets
            components (list): List of Component classes
            name (str): User Defined  Member Local Section Reduction Name
            comment (str, optional): Comments
            params (dict, optional): Any WS Parameter relevant to the object and its value in form of a dictionary
            model (RSTAB Class, optional): Model to be edited
        """

        # Client Model | Types For Timber Member Local Section Reduction
        clientObject = model.clientModel.factory.create('ns0:timber_member_local_section_reduction')

        # Clears object atributes | Sets all atributes to None
        clearAttributes(clientObject)

        # Local Section Reduction No.
        clientObject.no = no

        # Local Section Reduction Assigned Members
        clientObject.members = ConvertToDlString(members)

        # Local Section Reduction Assigned Member Sets
        clientObject.member_sets = ConvertToDlString(member_sets)

        # Local Section Reduction User defined Name
        if name:
            clientObject.user_defined_name_enabled = True
            clientObject.name = name

        # Local Section Reduction Components
        clientObject.components = model.clientModel.factory.create('ns0:array_of_timber_member_local_section_reduction_components')

        for i,j in enumerate(components):
            smlsr = model.clientModel.factory.create('ns0:timber_member_local_section_reduction_components_row')
            smlsr.no = i+1
            smlsr.row = model.clientModel.factory.create('ns0:timber_member_local_section_reduction_components')
            clearAttributes(smlsr.row)
            smlsr.row.reduction_type = components[i].reduction_type
            if smlsr.row.reduction_type == 'REDUCTION_COMPONENT_TYPE_RECTANGLE_OPENING':
                smlsr.row.position = components[i].position
                smlsr.row.width = components[i].width
                smlsr.row.height = components[i].height
                smlsr.row.z_axis_reference_type = components[i].z_axis_reference_type
                smlsr.row.distance = components[i].distance
                smlsr.row.multiple = components[i].multiple
                if smlsr.row.multiple:
                    smlsr.row.multiple_number = components[i].multiple_number
                    smlsr.row.multiple_offset_definition_type = components[i].multiple_offset_definition_type
                    smlsr.row.multiple_offset = components[i].multiple_offset
            elif smlsr.row.reduction_type == 'REDUCTION_COMPONENT_TYPE_CIRCLE_OPENING':
                smlsr.row.position = 1
                smlsr.row.z_axis_reference_type = components[i].z_axis_reference_type
                smlsr.row.distance = components[i].distance
                smlsr.row.diameter = components[i].diameter
                smlsr.row.multiple = components[i].multiple
                if smlsr.row.multiple:
                    smlsr.row.multiple_offset_definition_type = components[i].multiple_offset_definition_type
                    smlsr.row.multiple_number = components[i].multiple_number
                    smlsr.row.multiple_offset = components[i].multiple_offset
            elif smlsr.row.reduction_type == 'REDUCTION_COMPONENT_TYPE_START_NOTCH':
                smlsr.row.length = components[i].length
                smlsr.row.orientation_type = components[i].orientation_type
                smlsr.row.depth = components[i].depth
                smlsr.row.direction_type = components[i].direction_type
                smlsr.row.stability = components[i].stability
                smlsr.row.multiple = components[i].multiple
                smlsr.row.fire_design = components[i].fire_design
                smlsr.row.support = components[i].support
                if smlsr.row.fire_design:
                    smlsr.row.fire_exposure_top = components[i].fire_exposure_top
                    smlsr.row.fire_exposure_left = components[i].fire_exposure_left
                    smlsr.row.fire_exposure_right = components[i].fire_exposure_right
                    smlsr.row.fire_exposure_bottom = components[i].fire_exposure_bottom
                if smlsr.row.multiple:
                    smlsr.row.multiple_number = components[i].multiple_number
                    smlsr.row.multiple_offset_definition_type = components[i].multiple_offset_definition_type
                    smlsr.row.multiple_offset = components[i].multiple_offset
            elif smlsr.row.reduction_type == 'REDUCTION_COMPONENT_TYPE_INNER_NOTCH':
                smlsr.row.position = components[i].position
                smlsr.row.length = components[i].length
                smlsr.row.orientation_type = components[i].orientation_type
                smlsr.row.depth = components[i].depth
                smlsr.row.direction_type = components[i].direction_type
                smlsr.row.stability = components[i].stability
                smlsr.row.multiple = components[i].multiple
                smlsr.row.fire_design = components[i].fire_design
                if smlsr.row.fire_design:
                    smlsr.row.fire_exposure_top = components[i].fire_exposure_top
                    smlsr.row.fire_exposure_left = components[i].fire_exposure_left
                    smlsr.row.fire_exposure_right = components[i].fire_exposure_right
                    smlsr.row.fire_exposure_bottom = components[i].fire_exposure_bottom
                if smlsr.row.multiple:
                    smlsr.row.multiple_number = components[i].multiple_number
                    smlsr.row.multiple_offset_definition_type = components[i].multiple_offset_definition_type
                    smlsr.row.multiple_offset = components[i].multiple_offset
            elif smlsr.row.reduction_type == 'REDUCTION_COMPONENT_TYPE_END_NOTCH':
                smlsr.row.length = components[i].length
                smlsr.row.orientation_type = components[i].orientation_type
                smlsr.row.depth = components[i].depth
                smlsr.row.direction_type = components[i].direction_type
                smlsr.row.stability = components[i].stability
                smlsr.row.support = components[i].support
                smlsr.row.multiple = components[i].multiple
                smlsr.row.fire_design = components[i].fire_design
                if smlsr.row.fire_design:
                    smlsr.row.fire_exposure_top = components[i].fire_exposure_top
                    smlsr.row.fire_exposure_left = components[i].fire_exposure_left
                    smlsr.row.fire_exposure_right = components[i].fire_exposure_right
                    smlsr.row.fire_exposure_bottom = components[i].fire_exposure_bottom
            else:
                assert True, 'Unsupported reduction_type'

            clientObject.components.timber_member_local_section_reduction_components.append(smlsr)

        # Comment
        clientObject.comment = comment

        # Adding optional parameters via dictionary
        if params:
            for key in params:
                clientObject[key] = params[key]

        # Delete None attributes for improved performance
        deleteEmptyAttributes(clientObject)

        # Add Timber Member Local Section Reduction to Client Model
        model.clientModel.service.set_timber_member_local_section_reduction(clientObject)



from RSTAB.initModel import Model, clearAttributes, deleteEmptyAttributes, ConvertStrToListOfInt
from RSTAB.enums import ActionCategoryType, ActionType, ObjectTypes

class Action():
    def __init__(self,
                 no: int = 1,
                 action_category = ActionCategoryType.ACTION_CATEGORY_PERMANENT_G,
                 action_type = ActionType.ACTING_ALTERNATIVELY,
                 action_items: list = None,
                 name: str = '',
                 is_active: bool = True,
                 comment: str = '',
                 params: dict = None,
                 model = Model):
        '''
        Combination Wizard Action

        Args:
            no (int, optional): Action Tag
            action_category (enum, optional): Action Category Type Enumeration
            action_type (enum, optional): Action Type Enumeration
            action_items (list or list of lists, optional): Action Items
                if action_type == 'ACTING_DIFFERENTLY':
                    action_items = [[Load_case, acting_group], ...]
                else:
                    action_items = [Load_case, Load_case, ...]
            name (str, optional): User Defined Action Name
            is_active (bool, optional): Define if active
            comment (str, optional): Comments
            params (dict, optional): Any WS Parameter relevant to the object and its value in form of a dictionary
            model (RSTAB Class, optional): Model to be edited
        '''

        # Client model | Action
        clientObject = model.clientModel.factory.create('ns0:action')

        # Clears object atributes | Sets all atributes to None
        clearAttributes(clientObject)

        # Action No.
        clientObject.no = no

        # Action Category
        clientObject.action_category = action_category.name

        # Action Type
        clientObject.action_type = action_type.name

        # Action Items
        if action_items:
            for i in action_items:
                if action_type == ActionType.ACTING_DIFFERENTLY:
                    if not isinstance(i, list) or len(i) != 2:
                        ValueError('WARNING: Object Action, parameter action_items, size of the nested list must be 2.')
                    if isinstance(i[0], int) or isinstance(i[1], int):
                        pass
                    else:
                        ValueError('WARNING: Object Action, parameter action_items, items in the nested list must be integers.')
                else:
                    if not isinstance(i, int):
                        ValueError('WARNING: Object Action, parameter action_items must be list of integers.')

            items = model.clientModel.factory.create('ns0:array_of_action_items')
            for i,j in enumerate(action_items):
                item = model.clientModel.factory.create('ns0:action_items_row')
                item.no = i+1
                item.row = model.clientModel.factory.create('ns0:action_items')
                clearAttributes(item.row)
                if action_type == ActionType.ACTING_DIFFERENTLY:
                    item.row.load_case_item = j[0]
                    item.row.acting_group_number = j[1]
                else:
                    item.row.load_case_item = j

                items.action_items.append(item)

        # Active
        clientObject.is_active = is_active

        # Name
        if name:
            clientObject.user_defined_name_enabled = True
            clientObject.name = name

        # Comment
        clientObject.comment = comment

        # Adding optional parameters via dictionary
        if params:
            for key in params:
                clientObject[key] = params[key]

        # Clearing unused attributes
        deleteEmptyAttributes(clientObject)

        # Add Action to client model
        model.clientModel.service.set_action(clientObject)


    @staticmethod
    def DeleteAction(action_no: str = '1 2', model = Model):
        '''
        Delete Action objects

        Args:
            actions_no (str): Numbers of Actions to be deleted
            model (RSTAB Class, optional): Model to be edited
        '''

        # Delete Actions from client model
        for action in ConvertStrToListOfInt(action_no):
            model.clientModel.service.delete_object(ObjectTypes.E_OBJECT_TYPE_ACTION.name, action)

    @staticmethod
    def GetAction(object_index: int = 1, model = Model):

        '''
        Args:
            obejct_index (int): Action Index
            model (RSTAB Class, optional): Model to be edited
        '''

        # Get Action from client  model
        return model.clientModel.service.get_action(object_index)


from RSTAB.initModel import Model, clearAttributes, deleteEmptyAttributes, ConvertToDlString

class ImposedNodalDeformation():

    def __init__(self,
                 no: int = 1,
                 load_case_no: int = 1,
                 node_no: str = '1',
                 load_parameter: list = None,
                 comment: str = '',
                 params: dict = None,
                 model = Model):

        '''
        Args:
            no (int): Load Tag
            load_case_no (int): Assigned Load Case
            node_no (str): Assigned node(s)
            load_parameter (list): Load Parameters List
                load_parameter = [imposed_displacement_x, imposed_displacement_y, imposed_displacement_z, imposed_rotation_x, imposed_rotation_y imposed_rotation_z]
            comment (str, optional): Comments
            params (dict, optional): Any WS Parameter relevant to the object and its value in form of a dictionary
            model (RSTAB Class, optional): Model to be edited
        '''
        # Client model | Imposed Nodal Deformation
        clientObject = model.clientModel.factory.create('ns0:imposed_nodal_deformation')

        # Clears object attributes | Sets all attributes to None
        clearAttributes(clientObject)

        # Load No.
        clientObject.no = no

        # Load Case No.
        clientObject.load_case = load_case_no

        # Assigned Node No.
        clientObject.nodes = ConvertToDlString(node_no)

        # Load Parameter
        clientObject.imposed_displacement_x = load_parameter[0]
        clientObject.imposed_displacement_y = load_parameter[1]
        clientObject.imposed_displacement_z = load_parameter[2]

        clientObject.imposed_rotation_x = load_parameter[3]
        clientObject.imposed_rotation_y = load_parameter[4]
        clientObject.imposed_rotation_z = load_parameter[5]

        # Comment
        clientObject.comment = comment

        # Adding optional parameters via dictionary
        if params:
            for key in params:
                clientObject[key] = params[key]

        # Delete None attributes for improved performance
        deleteEmptyAttributes(clientObject)

        # Add Imposed Nodal Deformation to client model
        model.clientModel.service.set_imposed_nodal_deformation(load_case_no, clientObject)
